@@ -2,7 +2,7 @@
 #
 #     written by G. S. Marzot (marz@users.sourceforge.net)
 #
-#     Copyright (c) 2006-2011 SPARTA, Inc.  All rights reserved.
+#     Copyright (c) 2006-2012 SPARTA, Inc.  All rights reserved.
 #
 #     Copyright (c) 2006-2009 G. S. Marzot. All rights reserved.
 #
@@ -55,9 +55,7 @@ our @EXPORT = qw(
 	ns_t_nsec
 	ns_t_nsec3
 	ns_t_rrsig
-	p_as_error
 	VAL_AC_DS_MISSING
-	p_val_error
 	SOCKET
 	SR_CALL_ERROR
 	SR_DNS_GENERIC_ERROR
@@ -160,8 +158,6 @@ our @EXPORT = qw(
 	VAL_FROM_AUTHORITY
 	VAL_FROM_QUERY
 	VAL_FROM_UNSET
-	VAL_GETADDRINFO_HAS_STATUS
-	VAL_GETNAMEINFO_HAS_STATUS
 	VAL_IGNORE_VALIDATION
 	VAL_INCOMPLETE_PROOF
 	VAL_INTERNAL_ERROR
@@ -188,16 +184,18 @@ our @EXPORT = qw(
 	VAL_QFLAGS_NOCACHE_MASK
 	VAL_QFLAGS_USERMASK
 	VAL_QUERY_AC_DETAIL
-	VAL_QUERY_ASYNC
 	VAL_QUERY_DONT_VALIDATE
+	VAL_QUERY_NO_DLV
+	VAL_QUERY_USING_DLV
+	VAL_QUERY_ASYNC
+	VAL_QUERY_NO_EDNS0_FALLBACK
+	VAL_QUERY_SKIP_RESOLVER
+	VAL_QUERY_MARK_FOR_DELETION
+	VAL_QUERY_IGNORE_SKEW
+	VAL_QUERY_RECURSE
+	VAL_QUERY_SKIP_CACHE
 	VAL_QUERY_EDNS0_FALLBACK
 	VAL_QUERY_GLUE_REQUEST
-	VAL_QUERY_NO_DLV
-	VAL_QUERY_NO_EDNS0
-	VAL_QUERY_NO_EDNS0_FALLBACK
-	VAL_QUERY_RECURSE
-	VAL_QUERY_REFRESH_QCACHE
-	VAL_QUERY_USING_DLV
 	VAL_RESOURCE_UNAVAILABLE
 	VAL_SUCCESS
 	VAL_TRUSTED_ANSWER
