@@ -59,8 +59,10 @@ use strict;
 use Net::DNS::SEC::Tools::conf;
 use Fcntl qw(:DEFAULT :flock);
 
-our $VERSION = "1.9";
-our $MODULE_VERSION = "1.9.1";
+our $VERSION = "1.13";
+our $MODULE_VERSION = "1.13.0";
+
+#--------------------------------------------------------------------------
 
 our @ISA = qw(Exporter);
 
@@ -77,9 +79,12 @@ our @EXPORT = qw(keyrec_creat keyrec_open keyrec_read
 		 keyrec_signset_newname keyrec_signset_new keyrec_signset_keys
 		 keyrec_signset_addkey keyrec_signset_delkey
 		 keyrec_signset_haskey keyrec_signset_clear keyrec_signsets
+		 keyrec_signset_prefix
 		 keyrec_fmtchk
 	      );
 
+#--------------------------------------------------------------------------
+
 #
 # Fields in a zone keyrec.
 #
@@ -106,6 +111,16 @@ my @ZONEFIELDS = (
 			'lastset',
 			'gends',
 			'szopts',
+			'rollmgr',
+			'lastcmd',
+			'new_kgopts',
+			'new_ksklength',
+			'new_ksklife',
+			'new_random',
+			'new_revperiod',
+			'new_revtime',
+			'new_zsklength',
+			'new_zsklife',
 			'keyrec_signsecs',
 			'keyrec_signdate',
 		  );
@@ -142,6 +157,10 @@ my @KEYFIELDS = (
 		        'zonename',
 		 );
 
+#--------------------------------------------------------------------------
+
+my $DEFSETPREFIX = '-signset-';		# Default signing-set prefix.
+
 my $curkrfname;				# Name of open keyrec file.
 
 my @keyreclines;			# Keyrec lines.
@@ -571,37 +590,106 @@ sub keyrec_keypaths
 
 	#
 	# Get and verify the name of the appropriate signing set.
+	# We have to handle obsolete keys differently than other keys
+	# since obsolete keys don't have a reference in the zone keyrec.
 	#
-	return(@paths) if(!defined($keyrecs{$krn}{$krt}));
-	$sset = $keyrecs{$krn}{$krt};
+	if(($krt ne 'kskobs') && ($krt ne 'zskobs'))
+	{
+		return(@paths) if(!defined($keyrecs{$krn}{$krt}));
 
-	#
-	# Get and verify the key list.
-	#
-	return(@paths) if(!defined($keyrecs{$sset}{'keys'}));
-	$keylist = $keyrecs{$sset}{'keys'};
+		#
+		# Get and verify the key list.
+		#
+		$sset = $keyrecs{$krn}{$krt};
+
+		return(@paths) if(!defined($keyrecs{$sset}{'keys'}));
+		$keylist = $keyrecs{$sset}{'keys'};
+	}
+	else
+	{
+		foreach my $kname (keyrec_names())
+		{
+			next if($keyrecs{$kname}{'set_type'} ne $krt);
+			$keylist .= " $keyrecs{$kname}{'keys'}";
+		}
+	}
 
 	#
 	# Get the keys' paths and add 'em to the path array.
 	#
-	foreach my $kn (split /[\s,]/, $keylist)
+	foreach my $kn (sort(split /[\s,]/, $keylist))
 	{
-		#
-		# Verify that this key exists and is the right type.
-		#
-		next if(!defined($keyrecs{$kn}));
-		next if($keyrecs{$kn}{'keyrec_type'} ne $krt);
+		my @newpaths;
 
-		#
-		# Push the key's path onto the path list.
-		#
-		push @paths, $keyrecs{$kn}{'keypath'};
+		@newpaths = getsetpaths($kn,$krt);
+		push @paths, @newpaths if(@newpaths != 0);
 	}
 
 	#
 	# Return the path list.
 	#
-	return(sort(@paths));
+	return(@paths);
+}
+
+#--------------------------------------------------------------------------
+# Routine:	getsetpaths()
+#
+# Purpose:	Build a list of the keypaths for a given set.  This will
+#		fetch the paths for intermediate signing sets.
+#
+#		This routine is not exported.
+#
+sub getsetpaths
+{
+	my $kn = shift;					# Name of key to get.
+	my $kt = shift;					# Type of key to get.
+	my @paths = ();				# Set's path list.
+
+	#
+	# Verify that this key exists.
+	#
+	return(@paths) if($kn eq '') || (! defined($keyrecs{$kn}));
+
+	#
+	# If this is a set keyrec, we'll pull the key paths from its keylist.
+	# Otherwise, it's assumed to be a key keyrec and we'll take the key
+	# path from itself.
+	#
+	if($keyrecs{$kn}{'keyrec_type'} eq 'set')
+	{
+		my $keylist = $keyrecs{$kn}{'keys'};
+
+		#
+		# Get the keys' paths and add 'em to the path array.
+		#
+		foreach my $kname (split /[\s,]/, $keylist)
+		{
+			my @newpaths;
+			@newpaths = getsetpaths($kname,$kt);
+			push @paths, @newpaths if(@newpaths != 0);
+		}
+
+	}
+	else
+	{
+		#
+		# Verify that this key is the right type.
+		#
+		return(@paths) if($keyrecs{$kn}{'keyrec_type'} ne $kt);
+
+		#
+		# Push the key's path onto the path list.
+		#
+		if($keyrecs{$kn}{'keypath'} ne '')
+		{
+			push @paths, $keyrecs{$kn}{'keypath'};
+		}
+	}
+
+	#
+	# Return the path list.
+	#
+	return(@paths);
 }
 
 #--------------------------------------------------------------------------
@@ -1378,8 +1466,7 @@ sub keyrec_signset_newname
 	my $setname;				# Name of zone's last set.
 	my $oldind;				# Old index of zone's last set.
 	my $newind;				# New index of zone's last set.
-
-	my $setprefix = '-signset-';
+	my $ssprefix;				# Signing-set prefix.
 
 # print "keyrec_signset_newname($zone):  down in\n";
 
@@ -1389,11 +1476,16 @@ sub keyrec_signset_newname
 	$setname = $keyrecs{$zone}{'lastset'};
 	$setname = "$zone-signset-000" if(!defined($keyrecs{$zone}{'lastset'}));
 
+	#
+	# Build the signing-set prefix.
+	#
+	$ssprefix = keyrec_signset_prefix($zone);
+
 	#
 	# Get the first number in the set name.  If there isn't a number
 	# in the set name, we'll append a zero.
 	#
-	$setname =~ /$setprefix(\d+)/;
+	$setname =~ /$ssprefix(\d+)/;
 	$oldind = $1;
 	
 	if($oldind eq '')
@@ -1412,7 +1504,7 @@ sub keyrec_signset_newname
 		$newind = "0$newind";
 	}
 
-	$setname =~ s/$setprefix$oldind/$setprefix$newind/;
+	$setname =~ s/$ssprefix$oldind/$ssprefix$newind/;
 	keyrec_setval('zone',$zone,'lastset',$setname);
 
 	#
@@ -1756,6 +1848,21 @@ sub keyrec_signset_keys
 	return($keyrecs{$krname}{'keys'});
 }
 
+#--------------------------------------------------------------------------
+# Routine:	keyrec_signset_prefix()
+#
+# Purpose:	Build and return the signing-set prefix.
+#
+sub keyrec_signset_prefix
+{
+	my $zone = shift;			# Zone for this signing set.
+	my $prefix;				# Signing-set prefix.
+
+	$prefix = $zone . $DEFSETPREFIX;
+
+	return($prefix);
+}
+
 #--------------------------------------------------------------------------
 # Routine:	keyrec_init()
 #
@@ -2305,6 +2412,8 @@ Net::DNS::SEC::Tools::keyrec - DNSSEC-Tools I<keyrec> file operations
 
   @signset = keyrec_signsets();
 
+  $sset_prefix = keyrec_signset_prefix("example.com");
+
   keyrec_settime("zone","example.com");
   keyrec_settime("set","signing-set-42");
   keyrec_settime("key","Kexample.com.+005+76543");
@@ -2681,6 +2790,12 @@ following:  "kskcur", "kskpub", "kskrev", "kskobs", "zskcur", "zskpub",
 
 It returns 1 if the call is successful; 0 if it is not.
 
+=item I<keyrec_signset_prefix(zone_name)>
+
+I<keyrec_signset_prefix()> returns the signing set prefix formed by
+concatenating the zone name and $DEFSETPREFIX.  This prefix should be
+followed by a numeric index.
+
 =item I<keyrec_signset_addkey(signing_set_name,key_list)>
 
 I<keyrec_signset_addkey()> adds the keys listed in I<key_list> to the signing
@@ -2804,7 +2919,7 @@ See the COPYING file included with the DNSSEC-Tools package for details.
 
 =head1 AUTHOR
 
-Wayne Morrison, tewok@users.sourceforge.net
+Wayne Morrison, tewok@tislabs.com
 
 =head1 SEE ALSO
 
