@@ -96,8 +96,8 @@ use Net::DNS::SEC::Tools::conf;
 use Net::DNS::SEC::Tools::defaults;
 use Net::DNS::SEC::Tools::rolllog;
 
-our $VERSION = "1.9";
-our $MODULE_VERSION = "1.9.0";
+our $VERSION = "1.13";
+our $MODULE_VERSION = "1.13.0";
 
 our @ISA = qw(Exporter);
 
@@ -138,6 +138,7 @@ our @EXPORT = qw(
 			 ROLLCMD_MERGERRFS
 			 ROLLCMD_PHASEMSG
 			 ROLLCMD_ROLLALL
+			 ROLLCMD_ROLLALLKSKS
 			 ROLLCMD_ROLLALLZSKS
 			 ROLLCMD_ROLLKSK
 			 ROLLCMD_ROLLREC
@@ -147,6 +148,8 @@ our @EXPORT = qw(
 			 ROLLCMD_QUEUELIST
 			 ROLLCMD_QUEUESTATUS
 			 ROLLCMD_SHUTDOWN
+			 ROLLCMD_SIGNZONE
+			 ROLLCMD_SIGNZONES
 			 ROLLCMD_SKIPALL
 			 ROLLCMD_SKIPZONE
 			 ROLLCMD_SLEEPTIME
@@ -194,6 +197,16 @@ my $EOL		= "\015\012";			# Net-standard end-of-line.
 my $CHANNEL_TYPE = PF_UNIX;			# Type of channel we're using.
 my $UNIXSOCK	= "/rollmgr.socket";		# Unix socket name.
 
+#
+# Maximum lengths of Unix socket names for various systems.  These are used
+# in rollmgr_channel().
+#
+my $FREEBSD_MAXSOCKNAME	= 103;
+my $MACOSX_MAXSOCKNAME	= 103;
+my $LINUX_MAXSOCKNAME	= 107;
+my $UNKNOWN_MAXSOCKNAME = $MACOSX_MAXSOCKNAME;		# Use the shortest.
+my $maxsockname		= $UNKNOWN_MAXSOCKNAME;
+
 #
 # The CHANNEL_ entities are used for specifying whether rollmgr_sendcmd()
 # should or should not wait for a response from rollerd.
@@ -246,65 +259,71 @@ sub ROLLCMD_RC_ZSKROLL		{ return($ROLLCMD_RC_ZSKROLL);		};
 # The remaining ROLLCMD_ entities are the rollmgr_sendcmd() commands
 # recognized by rollerd.  %roll_commands is a hash table of valid commands.
 #
-my $ROLLCMD_DISPLAY	= "rollcmd_display";
-my $ROLLCMD_DSPUB	= "rollcmd_dspub";
-my $ROLLCMD_DSPUBALL	= "rollcmd_dspuball";
-my $ROLLCMD_GETSTATUS	= "rollcmd_getstatus";
-my $ROLLCMD_LOGFILE	= "rollcmd_logfile";
-my $ROLLCMD_LOGLEVEL	= "rollcmd_loglevel";
-my $ROLLCMD_LOGMSG	= "rollcmd_logmsg";
-my $ROLLCMD_LOGTZ	= "rollcmd_logtz";
-my $ROLLCMD_MERGERRFS	= "rollcmd_mergerrfs";
-my $ROLLCMD_PHASEMSG	= "rollcmd_phasemsg";
-my $ROLLCMD_ROLLALL	= "rollcmd_rollall";
-my $ROLLCMD_ROLLALLZSKS	= "rollcmd_rollallzsks";
-my $ROLLCMD_ROLLKSK	= "rollcmd_rollksk";
-my $ROLLCMD_ROLLREC	= "rollcmd_rollrec";
-my $ROLLCMD_ROLLZONE	= "rollcmd_rollzone";
-my $ROLLCMD_ROLLZSK	= "rollcmd_rollzsk";
-my $ROLLCMD_RUNQUEUE	= "rollcmd_runqueue";
-my $ROLLCMD_QUEUELIST	= "rollcmd_queuelist";
-my $ROLLCMD_QUEUESTATUS	= "rollcmd_queuestatus";
-my $ROLLCMD_SHUTDOWN	= "rollcmd_shutdown";
-my $ROLLCMD_SKIPALL	= "rollcmd_skipall";
-my $ROLLCMD_SKIPZONE	= "rollcmd_skipzone";
-my $ROLLCMD_SLEEPTIME	= "rollcmd_sleeptime";
-my $ROLLCMD_SPLITRRF	= "rollcmd_splitrrf";
-my $ROLLCMD_STATUS	= "rollcmd_status";
-my $ROLLCMD_ZONEGROUP	= "rollcmd_zonegroup";
-my $ROLLCMD_ZONELOG	= "rollcmd_zonelog";
-my $ROLLCMD_ZONESTATUS	= "rollcmd_zonestatus";
-my $ROLLCMD_ZSARGS	= "rollcmd_zsargs";
+my $ROLLCMD_DISPLAY	 = "rollcmd_display";
+my $ROLLCMD_DSPUB	 = "rollcmd_dspub";
+my $ROLLCMD_DSPUBALL	 = "rollcmd_dspuball";
+my $ROLLCMD_GETSTATUS	 = "rollcmd_getstatus";
+my $ROLLCMD_LOGFILE	 = "rollcmd_logfile";
+my $ROLLCMD_LOGLEVEL	 = "rollcmd_loglevel";
+my $ROLLCMD_LOGMSG	 = "rollcmd_logmsg";
+my $ROLLCMD_LOGTZ	 = "rollcmd_logtz";
+my $ROLLCMD_MERGERRFS	 = "rollcmd_mergerrfs";
+my $ROLLCMD_PHASEMSG	 = "rollcmd_phasemsg";
+my $ROLLCMD_ROLLALL	 = "rollcmd_rollall";
+my $ROLLCMD_ROLLALLKSKS	 = "rollcmd_rollallksks";
+my $ROLLCMD_ROLLALLZSKS	 = "rollcmd_rollallzsks";
+my $ROLLCMD_ROLLKSK	 = "rollcmd_rollksk";
+my $ROLLCMD_ROLLREC	 = "rollcmd_rollrec";
+my $ROLLCMD_ROLLZONE	 = "rollcmd_rollzone";
+my $ROLLCMD_ROLLZSK	 = "rollcmd_rollzsk";
+my $ROLLCMD_RUNQUEUE	 = "rollcmd_runqueue";
+my $ROLLCMD_QUEUELIST	 = "rollcmd_queuelist";
+my $ROLLCMD_QUEUESTATUS	 = "rollcmd_queuestatus";
+my $ROLLCMD_SHUTDOWN	 = "rollcmd_shutdown";
+my $ROLLCMD_SIGNZONE	 = "rollcmd_signzone";
+my $ROLLCMD_SIGNZONES	 = "rollcmd_signzones";
+my $ROLLCMD_SKIPALL	 = "rollcmd_skipall";
+my $ROLLCMD_SKIPZONE	 = "rollcmd_skipzone";
+my $ROLLCMD_SLEEPTIME	 = "rollcmd_sleeptime";
+my $ROLLCMD_SPLITRRF	 = "rollcmd_splitrrf";
+my $ROLLCMD_STATUS	 = "rollcmd_status";
+my $ROLLCMD_ZONEGROUP	 = "rollcmd_zonegroup";
+my $ROLLCMD_ZONELOG	 = "rollcmd_zonelog";
+my $ROLLCMD_ZONESTATUS	 = "rollcmd_zonestatus";
+my $ROLLCMD_ZSARGS	 = "rollcmd_zsargs";
 
-sub ROLLCMD_DISPLAY		{ return($ROLLCMD_DISPLAY);	};
-sub ROLLCMD_DSPUB		{ return($ROLLCMD_DSPUB);	};
-sub ROLLCMD_DSPUBALL		{ return($ROLLCMD_DSPUBALL);	};
-sub ROLLCMD_GETSTATUS		{ return($ROLLCMD_GETSTATUS);	};
-sub ROLLCMD_LOGFILE		{ return($ROLLCMD_LOGFILE);	};
-sub ROLLCMD_LOGLEVEL		{ return($ROLLCMD_LOGLEVEL);	};
-sub ROLLCMD_LOGMSG		{ return($ROLLCMD_LOGMSG);	};
-sub ROLLCMD_LOGTZ		{ return($ROLLCMD_LOGTZ);	};
-sub ROLLCMD_MERGERRFS		{ return($ROLLCMD_MERGERRFS);	};
-sub ROLLCMD_PHASEMSG		{ return($ROLLCMD_PHASEMSG);	};
-sub ROLLCMD_ROLLALL		{ return($ROLLCMD_ROLLALL);	};
-sub ROLLCMD_ROLLALLZSKS		{ return($ROLLCMD_ROLLALLZSKS);	};
-sub ROLLCMD_ROLLKSK		{ return($ROLLCMD_ROLLKSK);	};
-sub ROLLCMD_ROLLREC		{ return($ROLLCMD_ROLLREC);	};
-sub ROLLCMD_ROLLZONE		{ return($ROLLCMD_ROLLZONE);	};
-sub ROLLCMD_ROLLZSK		{ return($ROLLCMD_ROLLZSK);	};
-sub ROLLCMD_RUNQUEUE		{ return($ROLLCMD_RUNQUEUE);	};
-sub ROLLCMD_QUEUELIST		{ return($ROLLCMD_QUEUELIST);	};
-sub ROLLCMD_QUEUESTATUS		{ return($ROLLCMD_QUEUESTATUS);	};
-sub ROLLCMD_SHUTDOWN		{ return($ROLLCMD_SHUTDOWN);	};
-sub ROLLCMD_SKIPALL		{ return($ROLLCMD_SKIPALL);	};
-sub ROLLCMD_SKIPZONE		{ return($ROLLCMD_SKIPZONE);	};
-sub ROLLCMD_SLEEPTIME		{ return($ROLLCMD_SLEEPTIME);	};
-sub ROLLCMD_SPLITRRF		{ return($ROLLCMD_SPLITRRF);	};
-sub ROLLCMD_STATUS		{ return($ROLLCMD_STATUS);	};
-sub ROLLCMD_ZONEGROUP		{ return($ROLLCMD_ZONEGROUP);	};
-sub ROLLCMD_ZONELOG		{ return($ROLLCMD_ZONELOG);	};
-sub ROLLCMD_ZONESTATUS		{ return($ROLLCMD_ZONESTATUS);	};
-sub ROLLCMD_ZSARGS		{ return($ROLLCMD_ZSARGS);	};
+sub ROLLCMD_DISPLAY		{ return($ROLLCMD_DISPLAY);	 };
+sub ROLLCMD_DSPUB		{ return($ROLLCMD_DSPUB);	 };
+sub ROLLCMD_DSPUBALL		{ return($ROLLCMD_DSPUBALL);	 };
+sub ROLLCMD_GETSTATUS		{ return($ROLLCMD_GETSTATUS);	 };
+sub ROLLCMD_LOGFILE		{ return($ROLLCMD_LOGFILE);	 };
+sub ROLLCMD_LOGLEVEL		{ return($ROLLCMD_LOGLEVEL);	 };
+sub ROLLCMD_LOGMSG		{ return($ROLLCMD_LOGMSG);	 };
+sub ROLLCMD_LOGTZ		{ return($ROLLCMD_LOGTZ);	 };
+sub ROLLCMD_MERGERRFS		{ return($ROLLCMD_MERGERRFS);	 };
+sub ROLLCMD_PHASEMSG		{ return($ROLLCMD_PHASEMSG);	 };
+sub ROLLCMD_ROLLALL		{ return($ROLLCMD_ROLLALL);	 };
+sub ROLLCMD_ROLLALLKSKS		{ return($ROLLCMD_ROLLALLKSKS);	 };
+sub ROLLCMD_ROLLALLZSKS		{ return($ROLLCMD_ROLLALLZSKS);	 };
+sub ROLLCMD_ROLLKSK		{ return($ROLLCMD_ROLLKSK);	 };
+sub ROLLCMD_ROLLREC		{ return($ROLLCMD_ROLLREC);	 };
+sub ROLLCMD_ROLLZONE		{ return($ROLLCMD_ROLLZONE);	 };
+sub ROLLCMD_ROLLZSK		{ return($ROLLCMD_ROLLZSK);	 };
+sub ROLLCMD_RUNQUEUE		{ return($ROLLCMD_RUNQUEUE);	 };
+sub ROLLCMD_QUEUELIST		{ return($ROLLCMD_QUEUELIST);	 };
+sub ROLLCMD_QUEUESTATUS		{ return($ROLLCMD_QUEUESTATUS);	 };
+sub ROLLCMD_SHUTDOWN		{ return($ROLLCMD_SHUTDOWN);	 };
+sub ROLLCMD_SIGNZONE		{ return($ROLLCMD_SIGNZONE);	 };
+sub ROLLCMD_SIGNZONES		{ return($ROLLCMD_SIGNZONES);	 };
+sub ROLLCMD_SKIPALL		{ return($ROLLCMD_SKIPALL);	 };
+sub ROLLCMD_SKIPZONE		{ return($ROLLCMD_SKIPZONE);	 };
+sub ROLLCMD_SLEEPTIME		{ return($ROLLCMD_SLEEPTIME);	 };
+sub ROLLCMD_SPLITRRF		{ return($ROLLCMD_SPLITRRF);	 };
+sub ROLLCMD_STATUS		{ return($ROLLCMD_STATUS);	 };
+sub ROLLCMD_ZONEGROUP		{ return($ROLLCMD_ZONEGROUP);	 };
+sub ROLLCMD_ZONELOG		{ return($ROLLCMD_ZONELOG);	 };
+sub ROLLCMD_ZONESTATUS		{ return($ROLLCMD_ZONESTATUS);	 };
+sub ROLLCMD_ZSARGS		{ return($ROLLCMD_ZSARGS);	 };
 
 my $ROLLMGR_GROUP	= "g-";
 sub ROLLMGR_GROUP		{ return($ROLLMGR_GROUP);	};
@@ -323,6 +342,7 @@ my %roll_commands =
 	rollcmd_nodisplay	=> 1,
 	rollcmd_phasemsg	=> 1,
 	rollcmd_rollall		=> 1,
+	rollcmd_rollallksks	=> 1,
 	rollcmd_rollallzsks	=> 1,
 	rollcmd_rollksk		=> 1,
 	rollcmd_rollrec		=> 1,
@@ -332,6 +352,8 @@ my %roll_commands =
 	rollcmd_queuelist	=> 1,
 	rollcmd_queuestatus	=> 1,
 	rollcmd_shutdown	=> 1,
+	rollcmd_signzones	=> 1,
+	rollcmd_signzone	=> 1,
 	rollcmd_skipall		=> 1,
 	rollcmd_skipzone	=> 1,
 	rollcmd_sleeptime	=> 1,
@@ -550,6 +572,23 @@ sub rollmgr_prepdep
 		$osclass = "sysv";
 	}
 
+	#
+	# Set the maximum socket-name length depending on which operating
+	# system class we're running on.
+	#
+	if($osname eq "darwin")
+	{
+		$maxsockname = $MACOSX_MAXSOCKNAME;
+	}
+	elsif($osname eq "freebsd")
+	{
+		$maxsockname = $FREEBSD_MAXSOCKNAME;
+	}
+	elsif($osname eq "linux")
+	{
+		$maxsockname = $LINUX_MAXSOCKNAME;
+	}
+
 	#
 	# Get the appropriate switch table for this O/S class and save
 	# it for later reference.
@@ -1531,8 +1570,8 @@ sub rollmgr_channel
 		if($server)
 		{
 			setsockopt(SOCK,SOL_SOCKET,SO_REUSEADDR,pack("l",1));
-			bind(SOCK,sockaddr_in($CMDPORT,$ADDR)) || return(0);
-			listen(SOCK,SOMAXCONN) || return(0);
+			bind(SOCK,sockaddr_in($CMDPORT,$ADDR)) || return(-2);
+			listen(SOCK,SOMAXCONN) || return(-4);
 		}
 		else
 		{
@@ -1555,6 +1594,16 @@ sub rollmgr_channel
 		$unixsock = makelocalstatedir("/dnssec-tools") . $UNIXSOCK;
 # print STDERR "rollmgr_channel:  unixsock - <$unixsock>\n";
 
+		#
+		# Ensure the socket name isn't too long.  This is a result
+		# of a hardcode maximum length for socket names.  This is
+		# in the kernel and isn't 
+		#
+		return(-5) if(length($unixsock) > $maxsockname);
+
+		#
+		# Create the socket.
+		#
 		$sockdata = sockaddr_un($unixsock);
 
 		#
@@ -2120,6 +2169,16 @@ If I<serverflag> is false, then the client's side of the channel is created.
 Currently, the connection may only be made to the localhost.  This may be
 changed to allow remote connections, if this is found to be needed.
 
+Return Values:
+
+      1 - Communications channel successfully established.
+      0 - Unable to connect to the server.
+     -1 - Unable to create a Unix socket.
+     -2 - Unable to bind to the Unix socket. (server only)
+     -3 - Unable to change the permissions on the Unix socket. (server only)
+     -4 - Unable to listen on the Unix socket. (server only)
+     -5 - The socket name was longer than allowed for a Unix socket.
+
 =item I<rollmgr_queuecmd(cmdname, value)>
 
 This interface internally remembers a command and it's optional value
@@ -2185,6 +2244,8 @@ The available commands and their required data are:
    ROLLCMD_RUNQUEUE	none		rollerd runs through
 					its queue
    ROLLCMD_SHUTDOWN	none		stop rollerd
+   ROLLCMD_SIGNZONE	zone		sign a zone (no rollover)
+   ROLLCMD_SIGNZONEs	all or active	sign all or active zones
    ROLLCMD_SKIPALL	none		suspend all rollovers
    ROLLCMD_SKIPZONE	zone name	suspend rollover for a
 					rolling zone
@@ -2254,7 +2315,7 @@ See the COPYING file included with the DNSSEC-Tools package for details.
 
 =head1 AUTHOR
 
-Wayne Morrison, tewok@users.sourceforge.net
+Wayne Morrison, tewok@tislabs.com
 
 =head1 SEE ALSO
 
