@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 #
-# Copyright 2006-2011 SPARTA, Inc.  All rights reserved.  See the COPYING
+# Copyright 2006-2012 SPARTA, Inc.  All rights reserved.  See the COPYING
 # file distributed with this software for details.
 #
 # DNSSEC Tools
@@ -65,11 +65,17 @@ use Fcntl qw(:DEFAULT :flock);
 use Net::DNS::SEC::Tools::conf;
 use Net::DNS::SEC::Tools::rollmgr;
 
-our $VERSION = "1.9";
-our $MODULE_VERSION = "1.9.0";
+our $VERSION = "1.13";
+our $MODULE_VERSION = "1.13.0";
 
 our @ISA = qw(Exporter);
 
+#--------------------------------------------------------------------------
+
+#
+# Exported commands.
+#
+
 our @EXPORT = qw(
 			rollrec_add
 			rollrec_close
@@ -86,7 +92,6 @@ our @EXPORT = qw(
 			rollrec_lock
 			rollrec_merge
 			rollrec_names
-			rollrec_newrec
 			rollrec_read
 			rollrec_readfiles
 			rollrec_rectype
@@ -97,10 +102,27 @@ our @EXPORT = qw(
 			rollrec_split
 			rollrec_unlock
 			rollrec_write
+			rollrec_zonegroup
+			rollrec_zonegroups
+			rollrec_zonegroup_cmds
 		);
 
 #--------------------------------------------------------------------------
 
+#
+# Zonegroup commands.
+#
+my %zg_commands =
+	(
+		'rollcmd_dspub'		=>	1,
+		'rollcmd_rollksk'	=>	1,
+		'rollcmd_rollzone'	=>	1,
+		'rollcmd_rollzsk'	=>	1,
+		'rollcmd_skipzone'	=>	1,
+	);
+
+#--------------------------------------------------------------------------
+
 #
 # Default file names.
 #
@@ -115,6 +137,7 @@ my @ROLLFIELDS = (
 			'zonename',
 			'zonefile',
 			'keyrec',
+			'zonegroup',
 			'maxttl',
 			'administrator',
 			'directory',
@@ -137,7 +160,10 @@ my @ROLLFIELDS = (
 my @rollreclines;		# Rollrec lines.
 my $rollreclen;			# Number of rollrec lines.
 
-my %rollrecs;			# Rollrec hash table (keywords/values.)
+my %rollrecs;			# Rollrec hash table.
+my %rollrecindex;		# Maps rollrec names to @rollreclines indices.
+
+my %zonegroups;			# Zone group hash table.
 
 my $modified;			# File-modified flag.
 
@@ -497,26 +523,9 @@ sub rollrec_split
 		}
 
 		#
-		# Find this rollrec in @rollreclines.
+		# Find the index for this rollrec in @rollreclines.
 		#
-		for($rrind=0;$rrind<$rollreclen;$rrind++)
-		{
-			my $ln = $rollreclines[$rrind];	# Line in rollrec file.
-			my $rrname;			# Rollrec name.
-
-			#
-			# Dig out the line's keyword and value.
-			#
-			$ln =~ /^\s*(roll|skip)\s+"([a-z0-9\/\-+_.,: \t]+)"/i;
-			$rrname = $2;
-
-			#
-			# If this line has the rollrec's name and is the start
-			# of a new rollrec, then we've found our man.
-			#
-			#
-			last if(lc($rrname) eq lc($rrn));
-		}
+		$rrind = rrindex($rrn);
 
 		#
 		# Bump our count of split rollrecs.
@@ -553,6 +562,15 @@ sub rollrec_split
 			print ROLLREC_SPLIT "$ln";
 		}
 
+		#
+		# If we hit the beginning of the next rollrec or a
+		# blank line, we'll write a blank line and, drop out.
+		#
+		if($rrind == $rollreclen)
+		{
+			print ROLLREC_SPLIT "\n";
+		}
+
 		#
 		# Delete the named rollrec.
 		#
@@ -597,7 +615,8 @@ sub rollrec_readfile
 {
 	my $rfh = shift;			# File handle for rollrec file.
 	my $name;				# Name of the rollrec entry.
-	my $havecmdsalready = 0;
+	my $havecmdsalready = 0;		# Commands-read flag.
+	my $prevline = 'dummy';			# Previous line.
 
 	#
 	# If we already have commands loaded, don't reload them.
@@ -617,6 +636,15 @@ sub rollrec_readfile
 
 		$line = $_;
 
+		#
+		# Collapse consecutive blank lines to a single blank.
+		#
+		# This isn't strictly necessary, but it keeps rollrec files
+		# from getting filled with lots of blank lines.
+		#
+		next if(($prevline =~ /^\s*$/) && ($line =~ /^\s*$/));
+		$prevline = $line;
+
 		#
 		# Save the line in our array of rollrec lines.
 		#
@@ -662,7 +690,7 @@ sub rollrec_readfile
 				err("rollrec_readfile:  duplicate record name; aborting...\n",-1);
 				return(-1);
 			}
-			rollrec_newrec($keyword,$name);
+			rollrec_newrec($keyword,$name,$rollreclen - 1);
 			next;
 		}
 
@@ -672,7 +700,7 @@ sub rollrec_readfile
 			# The line is used to issue a specific command to run.
 			# We queue it for later processing.
 			#
-print STDERR "rollrec_readfile:  processing command: $value / $havecmdsalready\n";
+# print STDERR "rollrec_readfile:  processing command: $value / $havecmdsalready\n";
 
 			next if($havecmdsalready);
 
@@ -699,12 +727,35 @@ print STDERR "rollrec_readfile:  processing command: $value / $havecmdsalready\n
 			next;
 		}
 
+		#
+		# Save this zone's zonegroup.
+		#
+		if($keyword =~ /^zonegroup$/i)
+		{
+			$zonegroups{$value}++;
+		}
+
+
 		#
 		# Save this subfield into the rollrec's collection.
 		#
 		$rollrecs{$name}{$keyword} = $value;
 	}
 
+	#
+	# Make sure the last line is a blank line.
+	#
+	if($rollreclines[-1] !~ /^\s*$/)
+	{
+		push @rollreclines, "\n";
+		$rollreclen = @rollreclines;
+	}
+
+	#
+	# Build our index of rollrec indices.
+	#
+	buildrrindex();
+
 	return(0);
 }
 
@@ -728,6 +779,64 @@ sub rollrec_names
 	return(@names);
 }
 
+#--------------------------------------------------------------------------
+# Routine:	rollrec_zonegroups()
+#
+# Purpose:	Return the zonegroups hash.
+#
+sub rollrec_zonegroups
+{
+# print "rollrec_zonegroups:  down in\n";
+
+	return(%zonegroups);
+}
+
+#--------------------------------------------------------------------------
+# Routine:	rollrec_zonegroup()
+#
+# Purpose:	Return the zones (rollrec names) in a given zonegroup.
+#
+sub rollrec_zonegroup
+{
+	my $zgname = shift;				# Zonegroup to look up.
+	my @zglist = ();				# Zones in zonegroup.
+
+# print "rollrec_zonegroup:  down in\n";
+
+	#
+	# Return null if the name isn't defined.
+	#
+	return if(! defined($zgname));
+
+	#
+	# Build a list of the zones in this zonegroup.
+	#
+	foreach my $rrn (sort(keys(%rollrecs)))
+	{
+		if($rollrecs{$rrn}{'zonegroup'} eq $zgname)
+		{
+			push @zglist, $rrn;
+		}
+	}
+
+	#
+	# Return the list.
+	#
+	return(@zglist);
+}
+
+#--------------------------------------------------------------------------
+# Routine:	rollrec_zonegroup_cmds()
+#
+# Purpose:	Return the list of commands relevant to zonegroups.
+#
+sub rollrec_zonegroup_cmds
+{
+# print "rollrec_zonegroup_cmds:  down in\n";
+
+	return(%zg_commands);
+}
+
 #--------------------------------------------------------------------------
 # Routine:	rollrec_exists()
 #
@@ -787,6 +896,7 @@ sub rollrec_rectype
 {
 	my $name    = shift;		# Name of rollrec we're modifying.
 	my $rectype = shift;		# Rollrec's new type.
+	my $rrind;			# Rollrec's index.
 
 # print STDERR "rollrec_rectype:  <$name> <$rectype>\n";
 
@@ -796,42 +906,26 @@ sub rollrec_rectype
 	return(0) if(($rectype ne "roll") && ($rectype ne "skip"));
 
 	#
-	# Find the appropriate entry to modify in @rollreclines.  If the
-	# given field isn't set in $name's rollrec, we'll insert this as
-	# a new field at the end of that rollrec.
+	# Ensure this is a valid rollrec.
 	#
-	for(my $rrind=0;$rrind<$rollreclen;$rrind++)
-	{
-		my $line = $rollreclines[$rrind];	# Line in rollrec file.
-		my $rtype;				# Rollrec type.
-		my $rrname;				# Rollrec name.
-
-		#
-		# Dig out the line's keyword and value.
-		#
-		$line =~ /^\s*(roll|skip)\s+"([a-zA-Z0-9\/\-+_.,: \t]+)"/i;
-		$rtype = $1;
-		$rrname = $2;
-
-		#
-		# If this line has the rollrec's name and is the start of a
-		# new rollrec, then like the Mounties we've found our man.
-		# We'll change the record type and return success.
-		#
-		if(lc($rrname) eq lc($name))
-		{
-			$rollrecs{$name}{'rollrec_type'} = $rectype;
-			$line =~ s/$rtype/$rectype/;
-			$rollreclines[$rrind] = $line;
-			$modified = 1;
-			return(1);
-		}
-	}
+	return(0) if(!defined($rollrecindex{$name}));
 
 	#
-	# We didn't find the line, so we'll return failure.
+	# Get the rollrec's index.
 	#
-	return(0);
+	$rrind = rrindex($name);
+
+	#
+	# Change the rollrec's type.
+	#
+	$rollreclines[$rrind] =~ s/^(\s*)(roll|skip)(\s+)/$1$rectype$3/i;
+	$rollrecs{$name}{'rollrec_type'} = $rectype;
+
+	#
+	# Set the modified flag and return success.
+	#
+	$modified = 1;
+	return(1);
 }
 
 #--------------------------------------------------------------------------
@@ -865,7 +959,7 @@ sub rollrec_setval
 		#
 		# Add the rollrec to the %rollrecs hash.
 		#
-		rollrec_newrec("roll",$name);
+		rollrec_newrec("roll",$name,$rollreclen + 1);
 
 		#
 		# Start the new rollrec in @rollreclines.
@@ -873,6 +967,7 @@ sub rollrec_setval
 		$rollreclines[$rollreclen] = "\n";
 		$rollreclen++;
 		$rollreclines[$rollreclen] = "roll\t\"$name\"\n";
+		$rollrecindex{$name} = $rollreclen;
 		$rollreclen++;
 	}
 
@@ -882,32 +977,9 @@ sub rollrec_setval
 	$rollrecs{$name}{$field} = $val;
 
 	#
-	# Find the appropriate rollrec to modify in @rollreclines.  If the
-	# given field isn't set in $name's rollrec, we'll insert this as
-	# a new field at the end of that rollrec.
+	# Get the index of the rollrec in @rollreclines.
 	#
-	for($rrind=0;$rrind<$rollreclen;$rrind++)
-	{
-		my $line = $rollreclines[$rrind];	# Line in rollrec file.
-		my $rrname;				# Rollrec name.
-
-		#
-		# Dig out the line's keyword and value.
-		#
-		$line =~ /^\s*(roll|skip)\s+"([a-zA-Z0-9\/\-+_.,: \t]+)"/i;
-		$rrname = $2;
-
-		#
-		# If this line has the rollrec's name and is the start of a
-		# new rollrec, then we've found our man.
-		#
-		# IMPORTANT NOTE:  We will *always* find the rollrec we're
-		#		   looking for.  The exists() check above
-		#		   ensures that there will be a rollrec with
-		#		   the name we want.
-		#
-		last if(lc($rrname) eq lc($name));
-	}
+	$rrind = rrindex($name);
 
 	#
 	# Find the specified field's entry in the rollrec's lines in
@@ -993,6 +1065,11 @@ sub rollrec_setval
 		# Bump the array length counter.
 		#
 		$rollreclen++;
+
+		#
+		# Rebuild our table of rollrec indices.
+		#
+		buildrrindex();
 	}
 
 	#
@@ -1039,30 +1116,9 @@ sub rollrec_delfield
 	delete($rollrecs{$name}{$field});
 
 	#
-	# Find the appropriate rollrec to modify in @rollreclines.
+	# Get the index for the rollrec.
 	#
-	for($rrind=0;$rrind<$rollreclen;$rrind++)
-	{
-		my $line = $rollreclines[$rrind];	# Line in rollrec file.
-		my $rrname;				# Rollrec name.
-
-		#
-		# Dig out the line's keyword and value.
-		#
-		$line =~ /^\s*(roll|skip)\s+"([a-zA-Z0-9\/\-+_.,: \t]+)"/i;
-		$rrname = $2;
-
-		#
-		# If this line has the rollrec's name and is the start of a
-		# new rollrec, then we've found our man.
-		#
-		# IMPORTANT NOTE:  We will *always* find the rollrec we're
-		#		   looking for.  The exists() check above
-		#		   ensures that there will be a rollrec with
-		#		   the name we want.
-		#
-		last if(lc($rrname) eq lc($name));
-	}
+	$rrind = rrindex($name);
 
 	#
 	# Find the specified field's entry in the rollrec's lines in
@@ -1112,11 +1168,13 @@ sub rollrec_delfield
 	}
 
 	#
-	# If we found the entry, we'll delete it.
+	# If we found the entry, we'll delete it and rebuild the
+	# rollrec index table.
 	#
 	if($found)
 	{
 		splice @rollreclines, $fldind, 1;
+		buildrrindex();
 	}
 
 	#
@@ -1155,14 +1213,16 @@ sub rollrec_add
 	#
 	# Create the basic rollrec info.
 	#
-	rollrec_newrec($rrtype,$rrname);
+	rollrec_newrec($rrtype,$rrname,$rollreclen + 1);
 
 	#
 	# Add the new rollrec's first line to the end of the rollrec table.
+	# and add an entry to the rollrec index.
 	#
 	$rollreclines[$rollreclen] = "\n";
 	$rollreclen++;
 	$rollreclines[$rollreclen] = "roll\t\"$rrname\"\n";
+	$rollrecindex{$rrname} = $rollreclen;
 	$rollreclen++;
 
 	#
@@ -1255,9 +1315,10 @@ sub rollrec_del
 # print "rollrec_del:  down in\n";
 
 	#
-	# Don't allow empty rollrec names.
+	# Don't allow empty rollrec names or non-existent rollrecs.
 	#
 	return(-1) if($rrname eq "");
+	return(-1) if(!defined($rollrecindex{$rrname}));
 
 	#
 	# Get a copy of the rollrec from the rollrec hash and then delete
@@ -1268,41 +1329,24 @@ sub rollrec_del
 	delete $rollrecs{$rrname};
 
 	#
-	# Find the index of the first line for this rollrec in the
-	# list of file lines.
+	# Get the index for this rollrec.
 	#
-	for($ind = 0;$ind < $rollreclen; $ind++)
-	{
-		$line = $rollreclines[$ind];
-
-		$line =~ /\s*(\S+)\s+(\S+)/;
-		$lkey = $1;
-		$lval = $2;
-
-		$lval =~ s/"//g;
-
-		last if($lval eq $rrname);
-	}
-	$rrind = $ind;
-
-	#
-	# If we didn't find a rollrec with this name, return failure.
-	#
-	return(-1) if($ind == $rollreclen);
+	$rrind = rrindex($rrname);
 
 	#
 	# Find the beginning of the next rollrec.
 	#
-	for($ind = $rrind+1;$ind < $rollreclen; $ind++)
+	for($ind = $rrind+1; $ind < $rollreclen; $ind++)
 	{
 		$line = $rollreclines[$ind];
 
-		$line =~ /\s*(\S+)\s+(\S+)/;
+		$line =~ /^\s*(roll|skip)\s+"([a-zA-Z0-9\/\-+_.,: \t]+)"/i;
 		$lkey = $1;
 		$lval = $2;
 
-		last if($lkey eq "roll");
+		last if(($lkey eq "roll") || ($lkey eq "skip"));
 	}
+
 	$ind--;
 
 	#
@@ -1321,6 +1365,19 @@ sub rollrec_del
 	splice(@rollreclines,$rrind,$len);
 	$rollreclen -= $len;
 
+	#
+	# Fold two consecutive blank lines into one.
+	#
+	if(($rollreclines[$rrind-1] eq "\n") && ($rollreclines[$rrind] eq "\n"))
+	{
+		splice(@rollreclines,$rrind,1);
+	}
+
+	#
+	# Rebuild our table of rollrec indices.
+	#
+	buildrrindex();
+
 	#
 	# Mark that the file has been modified.
 	#
@@ -1378,32 +1435,20 @@ sub rollrec_rename
 	$rrr = $rollrecs{$newname};
 	%rollrec = %$rrr;
 
-	#
-	# Find the index of the first line for this rollrec in the
-	# list of file lines.
-	#
-	$rollreclen = @rollreclines;
-	for($rrind = 0;$rrind < $rollreclen; $rrind++)
-	{
-		$line = $rollreclines[$rrind];
-
-		$line =~ /^\s*([a-zA-Z_]+)\s+"([a-zA-Z0-9\/\-+_.,: \@\t]*)"/;
-		$lkey = $1;
-		$lval = $2;
-
-		$lval =~ s/"//g;
-
-		last if($lval eq $oldname);
-	}
-
 	#
 	# If we didn't find a rollrec with this name, return failure.
 	#
-	return(-5) if($rrind == $rollreclen);
+	return(-5) if(!defined($rollrecindex{name}));
+
+	#
+	# Get the index of this rollrec.
+	#
+	$rrind = rrindex($oldname);
 
 	#
 	# Find the beginning of the next rollrec.
 	#
+	$rollreclen = @rollreclines;
 	for($ind = $rrind+1;$ind < $rollreclen; $ind++)
 	{
 		$line = $rollreclines[$ind];
@@ -1444,9 +1489,11 @@ sub rollrec_rename
 	}
 
 	#
-	# Delete the old rollrec.
+	# Delete the old rollrec and the old name's entry in the index hash.
 	#
 	delete $rollrecs{$oldname};
+	$rollrecindex{$newname} = $rollrecindex{$oldname};
+	delete $rollrecindex{$oldname};
 
 	#
 	# Mark that the file has been modified.
@@ -1489,10 +1536,17 @@ sub rollrec_settime
 # Purpose:	Creates a rollrec in %rollrecs.  The name and type fields
 #		are set.
 #
+#		This routine is NOT rebuilding the index.  Doing this here
+#		has the potential for slowing down file reads and such.
+#		The index is dealt with in those module routines which call
+#		rollrec_newrec(); since it isn't exported, there should be
+#		no outside callers to worry about.
+#
 sub rollrec_newrec
 {
 	my $type = shift;		# Type of rollrec we're creating.
 	my $name = shift;		# Name of rollrec we're creating.
+	my $line = shift;		# Line number of this rollrec.
 
 # print "rollrec_newrec:  down in\n";
 
@@ -1500,6 +1554,8 @@ sub rollrec_newrec
 
 	$rollrecs{$name}{"rollrec_name"} = $name;
 	$rollrecs{$name}{"rollrec_type"} = $type;
+
+	$rollrecindex{$name} = $line;
 }
 
 #--------------------------------------------------------------------------
@@ -1550,6 +1606,8 @@ sub rollrec_init
 # print "rollrec_init:  down in\n";
 
 	%rollrecs     = ();
+	%zonegroups   = ();
+	%rollrecindex = ();
 	@rollreclines = ();
 	$rollreclen   = 0;
 	$modified     = 0;
@@ -1666,6 +1724,103 @@ sub rollrec_write
 	return(flock(ROLLREC,LOCK_UN));
 }
 
+#--------------------------------------------------------------------------
+# Routine:	buildrrindex()
+#
+# Purpose:	This routine builds a name->index hash table for the defined
+#		rollrecs.  The name is the name of a rollrec.  The index is
+#		that rollrec's index in the @rollreclines array.  This hash
+#		table drastically speeds up the reference time over the old
+#		method of searching the whole table.
+#
+sub buildrrindex
+{
+# print "buildrrindex:  down in\n";
+
+	#
+	# Zap the current rollrec index.
+	#
+	%rollrecindex = ();
+
+	#
+	# Traipse through @rollreclines, and save the line index for the
+	# start of each rollrec.
+	#
+	for(my $rrind=0;$rrind<$rollreclen;$rrind++)
+	{
+		my $line = $rollreclines[$rrind];	# Line in rollrec file.
+		my $rrname;				# Rollrec name.
+
+		#
+		# Dig out the line's keyword and value.
+		#
+		$line =~ /^\s*(roll|skip)\s+"([a-zA-Z0-9\/\-+_.,: \t]+)"/i;
+		$rrname = $2;
+
+		#
+		# If this is a roll or skip line, save the line index for
+		# this rollrec entry.
+		#
+		if($rrname ne '')
+		{
+			$rollrecindex{$rrname} = $rrind;
+		}
+	}
+}
+
+#--------------------------------------------------------------------------
+# Routine:	rrindex()
+#
+# Purpose:	Get the index for the named rollrec entry.  We'll first
+#		consult the name's existing index to @rollreclines.  If the
+#		name matches, all's well.  If it doesn't, then we'll rebuild
+#		the index hash and then get the table index.
+#
+#		Callers MUST have already checked that the rollrec name
+#		exists.  Arguably, this should be done here, but it isn't.
+#
+sub rrindex
+{
+	my $name = shift;			# Rollrec to look up.
+	my $rrind;				# Rollrec's index.
+	my $line;				# Supposed start of rollrec.
+	my $lname;				# Name from line.
+
+	#
+	# Find the index for this rollrec in @rollreclines.
+	#
+	$rrind = $rollrecindex{$name};
+
+	#
+	# Get the line we're expecting this rollrec to start with.
+	#
+	$line = $rollreclines[$rrind];
+
+	#
+	# Dig out the rollrec name from this line.
+	#
+	$line =~ /^\s*(roll|skip)\s+"([a-zA-Z0-9\/\-+_.,: \t]+)"/i;
+	$lname = $2;
+
+	#
+	# If this name doesn't match the requested name, we'll rebuild
+	# the index and call ourself to find the index.
+	#
+	# This should never fail, as long as the caller already ensured
+	# the rollrec name is valid.
+	#
+	if($lname ne $name)
+	{
+		buildrrindex();
+		$rrind = rrindex($name);
+	}
+
+	#
+	# Give the rollrec's index back to our caller.
+	#
+	return($rrind);
+}
+
 #--------------------------------------------------------------------------
 # Routine:	rollrec_dump_hash()
 #
@@ -1758,9 +1913,13 @@ Net::DNS::SEC::Tools::rollrec - Manipulate a DNSSEC-Tools rollrec file.
 
   $default_file = rollrec_default();
 
-  $count = rollrec_merge("primary.rrf", "new0.rrf", "new1".rrf");
+  $count = rollrec_merge("primary.rrf", "new0.rrf", "new1.rrf");
   @retvals = rollrec_split("new-rollrec.rrf", @rollrec_list);
 
+  %zgroups = rollrec_zonegroups();
+  @zgroup = rollrec_zonegroup($zonegroupname);
+  @zgcmds = rollrec_zonegroup_cmds();
+
   rollrec_write();
   rollrec_close();
   rollrec_discard();
@@ -1783,6 +1942,7 @@ of keyword/value entries.  The following is an example of a I<rollrec>:
 	zonename		"example.com"
 	zonefile		"/etc/dnssec-tools/zones/db.example.com"
 	keyrec			"/etc/dnssec-tools/keyrec/example.keyrec"
+	zonegroup		"example zones"
 	directory		"/etc/dnssec-tools/dir-example.com"
 	kskphase		"0"
 	zskphase		"2"
@@ -1816,7 +1976,7 @@ preserving formatting and comments.)
 After the file has been read, the contents are referenced using
 I<rollrec_fullrec()> and I<rollrec_recval()>.  The I<rollrec_add()>,
 I<rollrec_setval()>, and I<rollrec_settime()> interfaces are used
-to modify the contents of a {\it rollrec} record.
+to modify the contents of a I<rollrec> record.
 
 If the I<rollrec> file has been modified, it must be explicitly written or
 the changes will not saved.  I<rollrec_write()> saves the new contents to disk.
@@ -1825,6 +1985,11 @@ I<rollrec> file.  If a I<rollrec> file is no longer wanted to be open, yet
 the contents should not be saved, I<rollrec_discard()> gets rid of the data
 closes and the file handle B<without> saving any modified data.
 
+On reading a I<rollrec> file, consecutive blank lines are collapsed into a
+single blank line.  As I<rollrec> entries are added and deleted, files merged
+and files split, it is possible for blocks of consecutive blanks lines to
+grow.  This will prevent these blocks from growing excessively.
+
 =head1 ROLLREC LOCKING
 
 This module includes interfaces for synchronizing access to the I<rollrec>
@@ -1879,27 +2044,28 @@ A blank line is added after the final line of the new I<rollrec>.
 The I<rollrec> file is not written after I<rollrec_add()>, though
 it is internally marked as having been modified.
 
+=item I<rollrec_close()>
+
+This interface saves the internal version of the I<rollrec> file (opened with
+I<rollrec_read()>) and closes the file handle. 
+
 =item I<rollrec_del(rollrec_name)>
 
-This routine deletes a I<rollrec> from the I<rollrec> file and the internal
-representation of the file contents.  The I<rollrec> is deleted from both
+This routine deletes a I<rollrec> from the set of I<rollrec>s loaded into
+memory by the B<rollrec.pm> module.  The I<rollrec> is deleted from both
 the I<%rollrecs> hash table and the I<@rollreclines> array.
 
-Only the I<rollrec> itself is deleted from the file.  Any associated comments
-and blank lines surrounding it are left intact.  The I<rollrec> file is not
-written after I<rollrec_del()>, though it is internally marked as having been
-modified.
+The I<rollrec> file is not written after I<rollrec_del()>, though the
+collection of I<rollrec>s is internally marked as having been modified.
+
+Only the I<rollrec> itself is deleted.  Any associated comments and blank
+lines surrounding it are left intact.
 
 Return values are:
 
      0 successful rollrec deletion
     -1 unknown name
 
-=item I<rollrec_close()>
-
-This interface saves the internal version of the I<rollrec> file (opened with
-I<rollrec_read()>) and closes the file handle. 
-
 =item I<rollrec_delfield(rollrec_name,field)>
 
 Deletes the given field from the specified I<rollrec>.  The file is
@@ -1937,7 +2103,7 @@ I<rollrec> synchronization file does not exist, it will be created.  If the
 process can't create the synchronization file, an error will be returned.
 Success or failure is returned.
 
-=item I<rollrec_merge(target_rollrec_file,rollrec_file2, ... rollrec_fileN)>
+=item I<rollrec_merge(target_rollrec_file, rollrec_file1, ... rollrec_fileN)>
 
 This interface merges the specified I<rollrec> files.  It reads each file
 and parses them into a I<rollrec> hash table and a file-contents array.  The
@@ -2037,10 +2203,10 @@ will return the value "db.example.com".
 
 This routine renames the I<rollrec> named by I<old_rollrec_name> to
 I<new_rollrec_name>.  The actual effect is to change the name in the I<roll>
-or I<skip> line is changed to I<new_rollrec_name>.  The name is changed in
-the internal version of the the I<rollrec> file only.  The file itself is
-not changed, but must be saved by calling either I<rollrec_write()>,
-I<rollrec_save()>, or I<rollrec_saveas()>.
+or I<skip> line to I<new_rollrec_name>.  The name is changed in the internal
+version of the the I<rollrec> file only.  The file itself is not changed, but
+must be saved by calling either I<rollrec_write()>, I<rollrec_save()>, or
+I<rollrec_saveas()>.
 
 I<old_rollrec_name> must be the name of an existing I<rollrec>.  Conversely,
 I<new_rollrec_name> must not name an existing I<rollrec>.
@@ -2089,6 +2255,9 @@ current file to the file named in I<new_rollrec_file>.  If some of the given
 I<rollrec> names are invalid, the valid names will be moved to the new file
 and the invalid names will be returned in a list to the caller.
 
+Only the I<rollrec> entries themselves will be moved to the new I<rollrec>
+file.  Any associated comments will be left in the current I<rollrec> file.
+
 On success, the count of moved I<rollrec> entries is returned.  Error returns
 are given below.
 
@@ -2114,6 +2283,32 @@ rewritten.
 
 I<rollrec_write()> gets an exclusive lock on the I<rollrec> file while writing.
 
+=item I<rollrec_zonegroup($zonegroupname)>
+
+This interface returns a list of the zones in the zonegroup (named by
+I<$zonegroupname>) defined in the current I<rollrec> file.  Null is returned
+if there are no zones in the zonegroup.
+
+While this is using the term "zone", it is actually referring to the name of
+the rollrec entries.  For a particular rollrec entry, the rollrec name is
+usually the same as the zone name, but this is not a requirement.
+
+=item I<rollrec_zonegroup_cmds()>
+
+This interface returns a list of the rollctl commands whose behavior changes
+when they are used with the I<-group> option.
+
+=item I<rollrec_zonegroups()>
+
+This interface returns a hash table of the zonegroups defined in the current
+I<rollrec> file.  The hash key is the name of the zonegroup; the hash value
+is the number of zones in the zonegroup.  Null is returned if there are no
+zonegroups in the I<rollrec> file.
+
+While this is using the term "zone", it is actually referring to the name of
+the rollrec entries.  For a particular rollrec entry, the rollrec name is
+usually the same as the zone name, but this is not a requirement.
+
 =back
 
 =head1 ROLLREC INTERNAL INTERFACES
@@ -2132,12 +2327,6 @@ be lost.  An open I<rollrec> file handle will remain open, though the data are
 no longer held internally.  A new I<rollrec> file must be read in order to use
 the B<rollrec.pm> interfaces again.
 
-=item I<rollrec_newrec(type,name)>
-
-This interface creates a new I<rollrec>.  The I<rollrec_name> field in the
-I<rollrec> is set to the values of the I<name> parameter.  The I<type>
-parameter must be either "roll" or "skip".
-
 =item I<rollrec_default()>
 
 This routine returns the name of the default I<rollrec> file.
@@ -2173,12 +2362,12 @@ preserved as given in the I<rollrec> file.
 
 =head1 COPYRIGHT
 
-Copyright 2006-2011 SPARTA, Inc.  All rights reserved.
+Copyright 2006-2012 SPARTA, Inc.  All rights reserved.
 See the COPYING file included with the DNSSEC-Tools package for details.
 
 =head1 AUTHOR
 
-Wayne Morrison, tewok@users.sourceforge.net
+Wayne Morrison, tewok@tislabs.com
 
 =head1 SEE ALSO
 
