@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 #
-# Copyright 2005-2011 SPARTA, Inc.  All rights reserved.  See the COPYING
+# Copyright 2005-2012 SPARTA, Inc.  All rights reserved.  See the COPYING
 # file distributed with this software for details
 #
 # DNSSEC Tools
@@ -21,8 +21,8 @@ our @ISA = qw(Exporter);
 
 our @EXPORT = qw(timetrans fuzzytimetrans timetrans_weeks);
 
-our $VERSION = "1.9";
-our $MODULE_VERSION = "1.9.0";
+our $VERSION = "1.13";
+our $MODULE_VERSION = "1.13.0";
 
 
 #######################################################################
@@ -456,12 +456,12 @@ I<fuzzytimetrans(720000)> returns 1.2 weeks
 
 =head1 COPYRIGHT
 
-Copyright 2004-2011 SPARTA, Inc.  All rights reserved.
+Copyright 2004-2012 SPARTA, Inc.  All rights reserved.
 See the COPYING file included with the DNSSEC-Tools package for details.
 
 =head1 AUTHOR
 
-Wayne Morrison, tewok@users.sourceforge.net
+Wayne Morrison, tewok@tislabs.com
 
 =head1 SEE ALSO
 
