@@ -20,6 +20,8 @@
 
 #include <qdebug.h>
 
+#include <validator/validator.h>
+
 #if defined(Q_OS_SYMBIAN) && defined(ORIENTATIONLOCK)
 #include <eikenv.h>
 #include <eikappui.h>
@@ -100,7 +102,6 @@ void MainWindow::addAddress(QString server, int row) {
     QLabel *label;
     int column = 3;
 
-    qDebug() << "here: " << row << " / " << m_rows;
     if (row < 0) {
         qDebug() << "using mrows: " << m_rows;
         row = m_rows;
@@ -264,7 +265,7 @@ void MainWindow::getSubAnswers() {
 }
 
 bool
-MainWindow::doLookupTest(QString lookupName, int queryType, char *resolv_conf)
+MainWindow::doLookupTest(const QString lookupName, int queryType, char *resolv_conf)
 {
     val_status_t val_status;
     struct addrinfo *aitop = NULL;
@@ -278,12 +279,12 @@ MainWindow::doLookupTest(QString lookupName, int queryType, char *resolv_conf)
     }
 
     // perform the lookup
-    ret = val_res_query(context, lookupName.toUtf8(), ns_c_in,
+    ret = val_res_query(context, lookupName.toUtf8(), 1 /* ns_c_in */,
                         queryType, buf, sizeof(buf), &val_status);
     qDebug() << "here: lookingup=" << lookupName << ", ret=" << ret << " / " << val_status;
 
     if (aitop)
-        freeaddrinfo(aitop);
+        val_freeaddrinfo(aitop);
     if (context != NULL) {
         val_free_context(context);
     }
@@ -313,7 +314,7 @@ void MainWindow::busy() {
 void MainWindow::showAbout()
 {
     QMessageBox message;
-    message.setText("<p><b>DNSSEC-Check</b><p><i>DNSSEC-Tools Version: 1.12.2</i></p><p>DNSSEC-Check tests the likelyhood that your network will support client-side DNSSEC validation.  "
+    message.setText("<p><b>DNSSEC-Check</b><p><i>DNSSEC-Tools Version: 1.13</i></p><p>DNSSEC-Check tests the likelyhood that your network will support client-side DNSSEC validation.  "
                     "DNSSEC-Check is a application created for the <a href=\"http://www.dnssec-tools.org/\">DNSSEC-Tools</a> project."
                     "<p>This project is a work-in-progress and this is an alpha-version of this software.  It is currently most suited to people that "
                     "know and understand how the DNS and DNSSEC works."
@@ -371,7 +372,7 @@ void MainWindow::submitResults(QString locationDescription)
     }
 
     accessURL.addQueryItem("locationDescription", locationDescription);
-    accessURL.addQueryItem(".12.1");
+    accessURL.addQueryItem(".13");
 
     if (!m_manager) {
         m_manager = new QNetworkAccessManager();
