@@ -446,7 +446,7 @@ void GraphWidget::doActualLookup(const QString &lookupString, int lookupType)
         m_nodeList->reApplyFiltersTo(node);
 
     if (aitop)
-        freeaddrinfo(aitop);
+        val_freeaddrinfo(aitop);
     unbusy();
 }
 
@@ -653,7 +653,7 @@ void GraphWidget::about()
     msgBox.setWindowTitle("About DNSSEC-Nodes");
     msgBox.setText("<img src=\":/icons/dnssec-nodes-64x64.png\" style=\"float: right\"/>"
                    "<h1>DNSSEC-Nodes</h1><p><i>A component of the DNSSEC-Tools Project</i></p><p>For more information, please visit http://www.dnssec-tools.org/.</i></p>"
-                   "<p>DNSSEC-Tools Version: 1.12.2</p>");
+                   "<p>DNSSEC-Tools Version: 1.13</p>");
     msgBox.setStandardButtons(QMessageBox::Close);
     msgBox.exec();
 }
