@@ -15,11 +15,11 @@
  * WITH THE USE OR PERFORMANCE OF THE SOFTWARE.
  */
 /*
- * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 
-#include "validator-config.h"
+#include "validator/validator-config.h"
 #include <validator/validator.h>
 #include <validator/resolver.h>
 
@@ -204,7 +204,7 @@ main(int argc, char** argv)
     int flight = atoi(argv[3]);
     int numq = atoi(argv[4]);
 
-    res_io_set_debug(1);
+    res_set_debug_level(7);
 
     query_async_test(async, burst, flight, numq);
 
