@@ -11,17 +11,18 @@
 //     VAL_SUCCESS;
 */
 
-#include "validator-config.h"
+#include "validator/validator-config.h"
 #include <validator/validator.h>
 #include <validator/resolver.h>
 #include "validator_driver.h"
 
 typedef struct testcase_st {
     char               *desc;
-    char               *qn;
-    int                 qc;
-    int                 qt;
-    int                 qr[MAX_TEST_RESULTS];
+    char               *qn; /* name */
+    int                 qc; /* class */
+    int                 qt; /* type */
+    int                 qr[MAX_TEST_RESULTS]; /* expected rc */
+    struct timeval      start;
 #ifndef VAL_NO_ASYNC
     val_async_status   *as;
 #endif
@@ -32,11 +33,24 @@ typedef struct testcase_st {
 typedef struct testsuite_st {
     char                *name;
     testcase            *head;
-    int                 in_flight;
-    int                 remaining;
     struct testsuite_st *next;
 } testsuite;
 
+typedef struct testsuite_stats_st {
+    int                 in_flight;
+    int                 remaining;
+    int                 failed;
+} testsuite_stats;
+
+#ifndef VAL_NO_ASYNC
+typedef struct async_cbd_st {
+    val_context_t      *ctx;
+    testcase           *tc;
+    testsuite_stats    *ss;
+    int                 doprint;
+} async_cbd;
+#endif
+
 
 extern int             
 val_get_token(char **buf_ptr,
@@ -468,12 +482,12 @@ run_suite(val_context_t *context, testcase *curr_test, int tcs, int tce,
     int i, rc, run = 0;
     struct val_response resp;
 
-    memset(&resp, 0, sizeof(resp));
-
     for (i = tcs;
          curr_test != NULL && curr_test->desc != NULL && i <= tce;
          curr_test = curr_test->next) {
 
+        memset(&resp, 0, sizeof(resp));
+
         ++run;
         fprintf(stderr, "%d: ", ++i);
         rc = sendquery(context, curr_test->desc,
@@ -497,22 +511,62 @@ run_suite(val_context_t *context, testcase *curr_test, int tcs, int tce,
 
 #ifndef VAL_NO_ASYNC
 int
-suite_async_callback(val_async_status *async_status, int event,
+suite_async_callback(val_async_status *as, int event,
                      val_context_t *ctx, void *cb_data, val_cb_params_t *cbp)
 {
-    testsuite *suite;
+    async_cbd *acbd;
+    testcase  *tc;
     if ((NULL == cb_data) || (NULL == cbp)) {
         val_log(ctx, LOG_ERR, "bad parameter for callback");
         return VAL_BAD_ARGUMENT;
     }
 
-    suite = (testsuite *)cb_data;
-    --suite->in_flight;
-    --suite->remaining;
+    acbd = (async_cbd *)cb_data;
+    --acbd->ss->in_flight;
+    --acbd->ss->remaining;
+    tc = acbd->tc;
 
     val_log(ctx, LOG_INFO,
             "as 0x%x %s query completed; %d in flight, %d remaining",
-            async_status, cbp->name, suite->in_flight, suite->remaining);
+            as, cbp->name, acbd->ss->in_flight, acbd->ss->remaining);
+
+    if (cbp->retval == VAL_NO_ERROR) {
+
+        int ret_val = compose_answer(tc->qn, tc->qt, tc->qc, cbp->results,
+                                     &tc->resp);
+
+        if (VAL_NO_ERROR != ret_val) {
+            fprintf(stderr, "%s: \t", tc->desc);
+            fprintf(stderr, "FAILED: Error in compose_answer(): %d\n",
+                    ret_val);
+            ++acbd->ss->failed;
+        }
+        else {
+            if (tc->resp.vr_response == NULL) {
+                fprintf(stderr, "FAILED: No response\n");
+            } else if (acbd->doprint) {
+                print_val_response(&tc->resp);
+            }
+            FREE(tc->resp.vr_response);
+
+            ret_val = check_results(acbd->ctx, tc->desc, tc->qn, tc->qc, tc->qt,
+                                    tc->qr, cbp->results, 0, &tc->start);
+            if (0 != ret_val) {
+                ++acbd->ss->failed;
+            }
+        }
+
+        val_free_result_chain(cbp->results);
+        cbp->results = NULL;
+    } else {
+        fprintf(stderr, "%s: \t", tc->desc);
+        fprintf(stderr, "FAILED: Error during async resolution: %s\n",
+                p_val_err(cbp->retval));
+        ++acbd->ss->failed;
+    }
+
+    free(acbd);
+
     return VAL_NO_ERROR;
 }
 
@@ -525,18 +579,27 @@ run_suite_async(val_context_t *context, testsuite *suite, testcase *start_test,
     fd_set             activefds;
     struct timeval     timeout, now;
     testcase          *curr_test = start_test;
+    async_cbd         *acbd;
+    testsuite_stats    suite_stats, *sstats = &suite_stats;
 
     if (!curr_test || !suite)
         return 0;
 
-    i = tcs;
-    suite->remaining = tce - tcs + 1;
-    suite->in_flight = 0;
+    if (tcs > tce) {
+        fprintf(stderr,"bad range\n");
+        return 0;
+    }
 
-    while (suite->remaining || suite->in_flight) {
+    memset(sstats, 0x00, sizeof(sstats));
+    i = tcs;
+    sstats->remaining = tce - tcs + 1;
+    sstats->in_flight = 0;
+    sstats->failed = 0;
+
+    while (sstats->remaining) {
         /** send up to burst queries */
         for (j = 0;
-             suite->in_flight < max_in_flight &&
+             sstats->in_flight < max_in_flight &&
                  i <= tce &&
                  j < burst &&
                  curr_test;
@@ -544,63 +607,100 @@ run_suite_async(val_context_t *context, testsuite *suite, testcase *start_test,
             val_log(context, LOG_DEBUG, "starting test %i (max %d) %s", i, tce,
                     curr_test->desc);
             memset(&curr_test->resp, 0, sizeof(curr_test->resp));
+            acbd = (async_cbd*) MALLOC(sizeof(async_cbd));
+            acbd->ss = sstats;
+            acbd->tc = curr_test;
+            acbd->ctx = context;
+            acbd->doprint = doprint;
             rc = val_async_submit(context, curr_test->qn, curr_test->qc,
                                   curr_test->qt, flags, &suite_async_callback,
-                                  suite, &curr_test->as);
+                                  acbd, &curr_test->as);
             if ((rc != VAL_NO_ERROR) || (!curr_test->as)) {
-                val_log(context, LOG_ERR, "error sending %i %s", i,
-                        curr_test->desc);
+                val_log(context, LOG_ERR, "FAILED: error sending test %i: %s",
+                         i, curr_test->desc);
+                ++sstats->failed;
+                --sstats->remaining;
                 continue;
             }
+            gettimeofday(&curr_test->start, NULL);
             ++run;
-            ++suite->in_flight;
+            ++sstats->in_flight;
         }
         unsent = tce - i + 1;
 
         /** set up fdset/timeout for select */
         FD_ZERO(&activefds);
         nfds = 0;
-        timeout.tv_sec = LONG_MAX;
-        val_async_select_info(context, &activefds, &nfds, &timeout);
-        if (0 == nfds) {
-            val_log(context, LOG_DEBUG,
-                    "no file descriptors set! (%d unsent, %d inflight)",
-                    unsent, suite->in_flight);
-            /* maybe socket got closed, need to send next request */
-            rc = val_async_check(context, &activefds, &nfds, 0);
-            val_async_select_info(context, &activefds, &nfds, &timeout);
-            if (0 == nfds) {
-                if (suite->in_flight && !unsent) {
-                    val_log(context, LOG_DEBUG,
-                            "**** no file descriptors set! (%d unsent, %d inflight)",
-                            unsent, suite->in_flight);
-                    break;
-                }
-                continue;
-            }
-        }
-        /** adjust libsres absolute timeout to select relative timeout */
-        if (timeout.tv_sec == LONG_MAX)
-            timeout.tv_sec = 1;
-        else {
-            gettimeofday(&now, NULL);
-            if (timeout.tv_sec > now.tv_sec)
-                timeout.tv_sec -= now.tv_sec;
-            else
-                timeout.tv_sec = 0;
-        }
-
         /** don't sleep too long if more queries are waiting to be sent */
-        if (unsent && suite->in_flight < max_in_flight && timeout.tv_sec > 0) {
+        if (unsent && sstats->in_flight < max_in_flight && timeout.tv_sec > 0) {
             val_log(context, LOG_DEBUG,
                     "reducing timeout so we can send more requests");
             timeout.tv_sec = 0;
             timeout.tv_usec = 500;
+        } else {
+            timeout.tv_sec = 60; /* 1 min */
+            timeout.tv_usec = 0;
         }
+        val_log(context, LOG_INFO,
+                "timeout %ld.%ld, %d in flight, %d unsent %d remain",
+                timeout.tv_sec, timeout.tv_usec, sstats->in_flight, unsent,
+                sstats->remaining);
+        val_async_select_info(context, &activefds, &nfds, &timeout);
+        if (0 == nfds) {
+            val_log(context, LOG_DEBUG,
+                    "no file descriptors set! (%d unsent, %d inflight, %d remain)",
+                    unsent, sstats->in_flight, sstats->remaining);
+            /*
+             * maybe socket got closed & need to send next request,
+             * or answer is in cache and needs to be processed.
+             */
+            rc = val_async_check(context, &activefds, &nfds, 0);
+            if (0 == sstats->remaining)
+                break; /* all callbacks called */
+            val_log(context, LOG_DEBUG,
+                    "after check (%d unsent, %d inflight, %d remain)",
+                    unsent, sstats->in_flight, sstats->remaining);
+            val_async_select_info(context, &activefds, &nfds, &timeout);
+            if (0 == nfds) {
+                if (unsent) {
+                    if (sstats->in_flight < max_in_flight)
+                        continue; /* submit more if we can */
+                } else if (sstats->in_flight) {
+                    /*
+                     * all queries submitted, some in flight, but no fds
+                     * waiting? after we've called val_async_check()?
+                     * I don't think we can recover from that, but lets
+                     * try just in case.
+                     */
+                    int prev_inflight = sstats->in_flight;
+                    val_log(context, LOG_WARNING,
+                            "xxx file descriptors set! (%d inflight)",
+                            sstats->in_flight);
+                    while (sstats->in_flight) {
+                        /** try checking again */
+                        rc = val_async_check(context, &activefds, &nfds, 0);
+                        if (sstats->in_flight == prev_inflight)
+                            break; /* no progress */
+                        prev_inflight = sstats->in_flight;
+                        val_log(context, LOG_INFO,
+                                "xxx progress! (%d inflight)",
+                                sstats->in_flight);
+                    }
+                    if (nfds == 0) {
+                        sstats->failed += sstats->in_flight;
+                        break;
+                    }
+                    val_log(context, LOG_INFO,
+                            "xxx some file descriptors!" );
+                }
+            }
+        }
+
+        gettimeofday(&now, NULL);
         val_log(context, LOG_INFO,
                 "select @ %d, max fd %d, timeout %ld.%ld, %d in flight, %d unsent",
-                now.tv_sec, nfds, timeout.tv_sec, now.tv_usec,
-                suite->in_flight, unsent);
+                now.tv_sec, nfds, timeout.tv_sec, timeout.tv_usec,
+                sstats->in_flight, unsent);
         if ((nfds > 0) && (val_log_debug_level() >= LOG_DEBUG))
             res_io_count_ready(&activefds, nfds); // debug
 
@@ -625,6 +725,8 @@ run_suite_async(val_context_t *context, testsuite *suite, testcase *start_test,
 
     } /* while(remaining) */
 
+    *failed = sstats->failed;
+
     return run;
 }
 #endif /* ndef VAL_NO_ASYNC */
