@@ -6,6 +6,12 @@
 #ifndef _VALIDATOR_INTERNAL_H
 #define _VALIDATOR_INTERNAL_H
 
+#if !defined(WIN32) || defined(LIBVAL_CONFIGURED)
+#include "validator/validator-config.h"
+#else
+#include "validator/validator-compat.h"
+#endif
+
 #include "validator/validator.h"
 #include "validator/resolver.h"
 
@@ -285,7 +291,7 @@ extern          "C" {
     };
 
 
-    struct val_context {
+    struct libval_context {
 
 #ifndef VAL_NO_THREADS
         /*
