@@ -57,6 +57,12 @@
 #ifndef RESOLVER_H
 #define RESOLVER_H
 
+#include <sys/types.h>
+#include <sys/time.h>
+#ifdef __MINGW32__
+#include <winsock2.h>
+#endif
+
 #ifdef __cplusplus
 extern          "C" {
 #endif
@@ -206,6 +212,7 @@ int             get_tcp(const char *name_n,
                         struct name_server **server,
                     unsigned char ** response, size_t * response_length);
 void            print_response(unsigned char * ans, size_t resplen);
+int             res_gettimeofday_buf(char *buf, size_t bufsize);
 
 struct sockaddr_storage **create_nsaddr_array(int num_addrs);
 struct name_server *create_name_server(void);
@@ -304,6 +311,8 @@ res_async_ea_is_using_stream(struct expected_arrival *ea);
 int
 res_async_ea_isset(struct expected_arrival *ea, fd_set *fds);
 
+void res_switch_all_to_tcp_tid(int trans_id);
+
 /*
  * define timersub macro if not defined. Set top of this header file
  * for the license for this hunk of code.
@@ -343,7 +352,6 @@ res_async_ea_isset(struct expected_arrival *ea, fd_set *fds);
     } while (/* CONSTCOND */ 0)
 #endif
 
-
 #ifdef __cplusplus
 }                               /* extern C */
 #endif
