@@ -92,7 +92,6 @@
  * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
  * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
-#include "validator/validator-config.h"
 #include "validator-internal.h"
 
 #include "res_support.h"
@@ -178,7 +177,7 @@ do_section(ns_msg * handle, ns_sect section, int pflag, FILE * file)
     char            err_buf[ERRBUFLEN + 1];
 #endif
 
-    buf = MALLOC(buflen);
+    buf = (char *)MALLOC(buflen);
     if (buf == NULL) {
         res_log(NULL,LOG_DEBUG, ";; memory allocation failure\n");
         return;
