@@ -15,7 +15,6 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THE SOFTWARE.
  */
-#include "validator/validator-config.h"
 #include "validator-internal.h"
 
 #include "res_support.h"
@@ -494,31 +493,33 @@ res_nsfallback_ea(struct expected_arrival *ea, struct timeval *closest_event,
     long             i, old_size;
     struct expected_arrival *temp = ea;
 
-    if (!temp && !name)
+    if (!temp || !name)
         return -1;
 
-    if (!server) {
-        res_log(NULL, LOG_DEBUG, "libsres: ""no server specified");
-        // return -1;
-    }
-
-    for(;temp;temp=temp->ea_next) {
-        //res_print_ea(temp);
-        /** match name, then look for address */
-        if (namecmp(server->ns_name_n, temp->ea_ns->ns_name_n) != 0)
-            continue;
-        if (memcmp(server->ns_address[0],
-                   temp->ea_ns->ns_address[temp->ea_which_address],
-                   sizeof(*server->ns_address[0])) == 0)
-            break;
-    }
-
-    if (!temp) {
-        res_log(NULL, LOG_DEBUG, "libsres: "
-                "no matching server found for fallback");
+    if (!server && ea->ea_next) {
+        res_log(NULL, LOG_DEBUG,
+                "libsres: ""no server specified and more than one ea");
         return -1;
     }
 
+    if (server) {
+        for(;temp;temp=temp->ea_next) {
+            //res_print_ea(temp);
+            /** match name, then look for address */
+            if (namecmp(server->ns_name_n, temp->ea_ns->ns_name_n) != 0)
+                continue;
+            if (memcmp(server->ns_address[0],
+                       temp->ea_ns->ns_address[temp->ea_which_address],
+                       sizeof(*server->ns_address[0])) == 0)
+                break;
+        }
+        if (!temp) {
+            res_log(NULL, LOG_DEBUG, "libsres: "
+                    "no matching server found for fallback");
+            return -1;
+        }
+    }
+
     /** even if there is a smaller size to fall back to, no attempts left */
     if (temp->ea_remaining_attempts < 0) {
         res_log(NULL, LOG_DEBUG, "libsres: "
@@ -1230,10 +1231,10 @@ complete_read(SOCKET sock, u_char *field, size_t length)
         bytes = recv(sock, field + bytes_read, length - bytes_read, 0);
         if (bytes == SOCKET_ERROR) {
             bytes_read = -1;
-            break;
+            return -1;
         }
         bytes_read += bytes;
-    } while (bytes_read < length);
+    } while (bytes_read < length && bytes != 0);
 
     return bytes_read;
 }
@@ -1382,7 +1383,9 @@ res_switch_to_tcp(struct expected_arrival *ea)
     if (NULL == ea)
         return;
 
-    FREE(ea->ea_response);
+    if (ea->ea_response != NULL) {
+        FREE(ea->ea_response);
+    }
     ea->ea_response = NULL;
     ea->ea_response_length = 0;
 
@@ -1412,7 +1415,9 @@ res_switch_all_to_tcp(struct expected_arrival *ea)
 
     for (; ea; ea = ea->ea_next) {
 
-        FREE(ea->ea_response);
+        if (ea->ea_response != NULL) {
+            FREE(ea->ea_response);
+        }
         ea->ea_response = NULL;
         ea->ea_response_length = 0;
 
