@@ -19,7 +19,6 @@
  * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
-#include "validator/validator-config.h"
 #include "validator-internal.h"
 
 #include <openssl/rand.h>
@@ -34,7 +33,7 @@ static int      seq_number = 0;
 FILE           *logfile = NULL;
 #define MEM_LOGFILE "memory_logfile"
 
-#ifndef HAVE_GETTIMEOFDAY
+#ifdef WIN32
 /*
    Implementation as per:
    The Open Group Base Specifications, Issue 6
@@ -57,6 +56,40 @@ int gettimeofday(struct timeval* p, void* tz /* IGNORED */)
 }
 #endif
 
+int
+res_gettimeofday_buf(char *buf, size_t bufsize) {
+
+    struct timeval  tv;
+    struct tm       *tp;
+    struct tm       tm;
+
+    tp = NULL;
+
+#ifdef WIN32
+    time(&tv.tv_sec);
+#else
+    gettimeofday(&tv, NULL);
+#endif
+
+#ifdef HAVE_LOCALTIME_R
+    localtime_r(&tv.tv_sec, &tm);
+    tp = &tm;
+#else
+    tp = localtime(&tv.tv_sec);
+#endif
+
+    if (tp) {
+        snprintf(buf, bufsize, "%04d%02d%02d::%02d:%02d:%02d ", 
+            tp->tm_year+1900, tp->tm_mon+1, tp->tm_mday,
+            tp->tm_hour, tp->tm_min, tp->tm_sec);
+    } else {
+        snprintf(buf, bufsize, "0000:00:00::00:00:00 ");
+    }
+
+    return 0;
+}
+
+
 void
 my_free(void *p, char *filename, int lineno)
 {
@@ -233,11 +266,11 @@ parse_name_server(const char *cp, const char *name_n)
     struct sockaddr_in *sin = (struct sockaddr_in *)&serv_addr;
 #ifdef VAL_IPV6
     struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *)&serv_addr;
-#if defined( WIN32 ) && !defined( LIBVAL_USE_WOCK )
+#if defined( WIN32 )
     size_t addrlen6 = sizeof(struct sockaddr_in6);
 #endif
 #endif
-#if defined( WIN32 ) && !defined( LIBVAL_USE_WOCK )
+#if defined( WIN32 )
     size_t addrlen4 = sizeof(struct sockaddr_in);
 #endif
 
@@ -584,19 +617,12 @@ void
 res_log(void *dont_care, int level, const char *template, ...)
 {
     char            buf[1028];
-    struct timeval  tv;
-    struct tm       *tm;
     va_list         ap;
 
     if (NULL == template || level > sres_level)
         return;
 
-    gettimeofday(&tv, NULL);
-    tm = localtime(&tv.tv_sec);
-
-    snprintf(buf, sizeof(buf) - 2, "%04d%02d%02d::%02d:%02d:%02d ", 
-            tm->tm_year+1900, tm->tm_mon+1, tm->tm_mday,
-            tm->tm_hour, tm->tm_min, tm->tm_sec);
+    res_gettimeofday_buf(buf, sizeof(buf) - 2);
     va_start(ap, template);
     vsnprintf(&buf[19], sizeof(buf) - 21, template, ap);
     va_end(ap);
@@ -609,18 +635,11 @@ void
 res_log_ap(void *dont_care, int level, const char *template, va_list ap)
 {
     char            buf[1028];
-    struct timeval  tv;
-    struct tm       *tm;
 
     if (NULL == template || level > sres_level)
         return;
 
-    gettimeofday(&tv, NULL);
-    tm = localtime(&tv.tv_sec);
-
-    snprintf(buf, sizeof(buf) - 2, "%04d%02d%02d::%02d:%02d:%02d ", 
-            tm->tm_year+1900, tm->tm_mon+1, tm->tm_mday,
-            tm->tm_hour, tm->tm_min, tm->tm_sec);
+    res_gettimeofday_buf(buf, sizeof(buf) - 2);
     vsnprintf(&buf[19], sizeof(buf) - 21, template, ap);
 
     fprintf(stderr, "%s\n", buf);
