@@ -1,35 +1,10 @@
 /*
- * Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 #ifndef VAL_ASSERTION_H
 #define VAL_ASSERTION_H
 
-
-#ifndef VAL_NO_THREADS
- 
-#define LOCK_QC_SH(qc) \
-    pthread_rwlock_rdlock(&qc->qc_rwlock)
-
-#define TRY_LOCK_QC_EX(qc) \
-   (0 == pthread_rwlock_trywrlock(&qc->qc_rwlock)) 
-
-#define LOCK_QC_EX(qc) \
-    pthread_rwlock_wrlock(&qc->qc_rwlock)
-
-#define UNLOCK_QC(qc) \
-    pthread_rwlock_unlock(&qc->qc_rwlock)
-
-#else
-
-#define LOCK_QC_SH(qc) 
-#define TRY_LOCK_QC_EX(qc) (1 == 1)
-#define LOCK_QC_EX(qc) 
-#define UNLOCK_QC(qc) 
-
-#endif /*VAL_NO_THREADS*/
-
-
 struct nsecprooflist {
     struct rrset_rec *the_set;
     struct val_internal_result *res;
