@@ -24,7 +24,6 @@
  * Contains implementation for storage and retrieval functions for the validator
  * rrset cache.
  */
-#include "validator/validator-config.h"
 #include "validator-internal.h"
 
 #include "val_support.h"
@@ -43,7 +42,6 @@ struct zone_ns_map_t {
  */
 static struct rrset_rec *unchecked_ns_info = NULL;
 static struct rrset_rec *unchecked_answers = NULL;
-static struct rrset_rec *unchecked_proofs = NULL;
 
 /*
  * Also maintain mapping between zone and name server, 
@@ -60,8 +58,6 @@ static pthread_rwlock_t ns_rwlock;
 static int ns_rwlock_init = -1;
 static pthread_rwlock_t ans_rwlock;
 static int ans_rwlock_init = -1;
-static pthread_rwlock_t proof_rwlock;
-static int proof_rwlock_init = -1;
 static pthread_rwlock_t map_rwlock;
 static int map_rwlock_init = -1;
 
@@ -119,7 +115,17 @@ stow_info(struct rrset_rec **unchecked_info, struct rrset_rec **new_info, struct
     while (*new_info) {
         new_rr = *new_info;
         delete_newrr = 0;
-        if (!IN_BAILIWICK(new_rr->rrs_name_n, matched_q)) {
+        if (!IN_BAILIWICK(new_rr->rrs_name_n, matched_q) ||
+            /* 
+             * no need to save any negative response
+             * meta-data other than ns_t_soa since 
+             * we will never look for these record types in
+             * our cache.
+             */
+#ifdef LIBVAL_NSEC3
+            new_rr->rrs_type_h == ns_t_nsec3 ||
+#endif
+            new_rr->rrs_type_h == ns_t_nsec) {
             delete_newrr = 1;
         } else {
           old = *unchecked_info;
@@ -200,9 +206,6 @@ get_cached_rrset(struct val_query_chain *matched_q,
     u_int16_t class_h;
     u_char *name_n;
 
-    int look_for_negative;
-    int done;
-
     if (!matched_q || !response)
         return VAL_BAD_ARGUMENT;
 
@@ -214,86 +217,65 @@ get_cached_rrset(struct val_query_chain *matched_q,
 
     gettimeofday(&tv, NULL);
 
-    look_for_negative = 0;
-    done = 0;
-
-    while (!done) {
-
-        if (look_for_negative) {
+    if (type_h == ns_t_ns) {
 #ifndef VAL_NO_THREADS
-            lk = &proof_rwlock;
-            VAL_CACHE_LOCK_INIT(lk, proof_rwlock_init);
-            VAL_CACHE_LOCK_SH(lk);
+        lk = &ns_rwlock;
+        VAL_CACHE_LOCK_INIT(lk, ns_rwlock_init);
+        VAL_CACHE_LOCK_SH(lk);
 #endif /* VAL_NO_THREADS */
-            answer_head = &unchecked_proofs;
-        } else if (type_h == ns_t_ns) {
+        answer_head = &unchecked_ns_info;
+    } else {
 #ifndef VAL_NO_THREADS
-            lk = &ns_rwlock;
-            VAL_CACHE_LOCK_INIT(lk, ns_rwlock_init);
-            VAL_CACHE_LOCK_SH(lk);
+        lk = &ans_rwlock;
+        VAL_CACHE_LOCK_INIT(lk, ans_rwlock_init);
+        VAL_CACHE_LOCK_SH(lk);
 #endif /* VAL_NO_THREADS */
-            answer_head = &unchecked_ns_info;
-        } else {
-#ifndef VAL_NO_THREADS
-            lk = &ans_rwlock;
-            VAL_CACHE_LOCK_INIT(lk, ans_rwlock_init);
-            VAL_CACHE_LOCK_SH(lk);
-#endif /* VAL_NO_THREADS */
-            answer_head = &unchecked_answers;
-        }
+        answer_head = &unchecked_answers;
+    }
 
-        prev = NULL;
-        new_answer = NULL;
-        if (answer_head) 
-            next_answer = *answer_head;
-        else
-            next_answer = NULL;
+    prev = NULL;
+    new_answer = NULL;
+    if (answer_head) 
+        next_answer = *answer_head;
+    else
+        next_answer = NULL;
     
-        while (next_answer) {
+    while (next_answer) {
 
-            if (tv.tv_sec < next_answer->rrs_ttl_x &&
-                next_answer->rrs_class_h == class_h) {
+        if (tv.tv_sec < next_answer->rrs_ttl_x &&
+            next_answer->rrs_class_h == class_h) {
 
-                /* if matching type or cname indirection */
-                if (((next_answer->rrs_type_h == type_h ||
-                    (next_answer->rrs_type_h == ns_t_cname &&
-                    ALIAS_MATCH_TYPE(type_h))) &&
-                    /* and name is an exact match */
-                    (namecmp(next_answer->rrs_name_n, name_n) == 0)) ||
-                    /* OR */
-                    /* DNAME indirection */
-                    ((next_answer->rrs_type_h == ns_t_dname &&
-                    ALIAS_MATCH_TYPE(type_h)) &&
-                    /* and name applies */
-                    (NULL != (u_char *) namename(name_n, 
+            /* if matching type or cname indirection */
+            if (((next_answer->rrs_type_h == type_h ||
+                (next_answer->rrs_type_h == ns_t_cname &&
+                ALIAS_MATCH_TYPE(type_h))) &&
+                /* and name is an exact match */
+                (namecmp(next_answer->rrs_name_n, name_n) == 0)) ||
+                /* OR */
+                /* DNAME indirection */
+                ((next_answer->rrs_type_h == ns_t_dname &&
+                ALIAS_MATCH_TYPE(type_h)) &&
+                /* and name applies */
+                (NULL != (u_char *) namename(name_n, 
                                     next_answer->rrs_name_n)))) {
 
-                    if (next_answer->rrs_data != NULL) {
-                        new_answer = copy_rrset_rec(next_answer);
-                        if (new_answer) {
-                            /* Adjust the TTL */
-                            new_answer->rrs_ttl_h = next_answer->rrs_ttl_x - tv.tv_sec; 
-                        }
-                        break;
+                if (next_answer->rrs_data != NULL) {
+                    new_answer = copy_rrset_rec(next_answer);
+                    if (new_answer) {
+                        /* Adjust the TTL */
+                        new_answer->rrs_ttl_h = next_answer->rrs_ttl_x - tv.tv_sec; 
                     }
-                } 
-            }
-
-            prev = next_answer;
-            next_answer = next_answer->rrs_next;
+                    break;
+                }
+            } 
         }
 
-        VAL_CACHE_UNLOCK(lk);
-
-        if (!new_answer && type_h == ns_t_soa && !look_for_negative) {
-            /* look for soa in negative cache */
-            done = 0;
-            look_for_negative = 1;
-        } else {
-            done = 1;
-        }
+        prev = next_answer;
+        next_answer = next_answer->rrs_next;
     }
 
+    VAL_CACHE_UNLOCK(lk);
+
     /* Construct the response */
     if (new_answer) {
         char *name_p;
@@ -400,19 +382,6 @@ stow_answers(struct rrset_rec **new_info, struct val_query_chain *matched_q)
     return rc;
 }
 
-int
-stow_negative_answers(struct rrset_rec **new_info, struct val_query_chain *matched_q)
-{
-    int             rc;
-
-    VAL_CACHE_LOCK_INIT(&proof_rwlock, proof_rwlock_init);
-    VAL_CACHE_LOCK_EX(&proof_rwlock);
-    rc = stow_info(&unchecked_proofs, new_info, matched_q);
-    VAL_CACHE_UNLOCK(&proof_rwlock);
-
-    return rc;
-}
-
 /*
  * Maintain a mapping between the zone and the name server that answered 
  * data for it 
@@ -632,12 +601,6 @@ free_validator_cache(void)
     unchecked_answers = NULL;
     VAL_CACHE_UNLOCK(&ans_rwlock);
     
-    VAL_CACHE_LOCK_INIT(&proof_rwlock, proof_rwlock_init);
-    VAL_CACHE_LOCK_EX(&proof_rwlock);
-    res_sq_free_rrset_recs(&unchecked_proofs);
-    unchecked_proofs = NULL;
-    VAL_CACHE_UNLOCK(&proof_rwlock);
-
     free_zone_nslist();
 
     return VAL_NO_ERROR;
