@@ -12,20 +12,17 @@
  * DESCRIPTION
  * Contains implementation of val_getaddrinfo() and friends
  */
-#include "validator/validator-config.h"
 #include "validator-internal.h"
 
 #include "val_policy.h"
 #include "val_parse.h"
 #include "val_context.h"
 
-#if !HAVE_FREEADDRINFO
-
 /* 
- * define this freeaddrinfo if we don't have it already available
+ * Free the addrinfo structure that we have allocated 
  */
 void
-freeaddrinfo(struct addrinfo *ainfo)
+val_freeaddrinfo(struct addrinfo *ainfo)
 {
     struct addrinfo *acurr = ainfo;
 
@@ -42,8 +39,6 @@ freeaddrinfo(struct addrinfo *ainfo)
     }
 }
 
-#endif
-
 /*
  * Function: append_addrinfo
  *
@@ -332,7 +327,7 @@ process_service_and_hints(const char *servname,
          */
         if (created_locally) {
             *res = NULL;
-            freeaddrinfo(a1);
+            val_freeaddrinfo(a1);
         }
         return EAI_SERVICE;
     }
@@ -382,12 +377,12 @@ get_addrinfo_from_etc_hosts(val_context_t * ctx,
     while (hs) {
         int             alias_index = 0;
         struct sockaddr_in  sa;
-#if defined( WIN32 ) && !defined( LIBVAL_USE_WOCK )
+#if defined( WIN32 )
         size_t addrlen4 = sizeof(struct sockaddr_in);
 #endif
 #ifdef VAL_IPV6
         struct sockaddr_in6 sa6;
-#if defined( WIN32 ) && !defined( LIBVAL_USE_WOCK )
+#if defined( WIN32 )
         size_t addrlen6 = sizeof(struct sockaddr_in6);
 #endif
 #endif
@@ -398,7 +393,7 @@ get_addrinfo_from_etc_hosts(val_context_t * ctx,
             (struct addrinfo *) malloc(sizeof(struct addrinfo));
         if (!ainfo) {
             if (retval)
-                freeaddrinfo(retval);
+                val_freeaddrinfo(retval);
             return EAI_MEMORY;
         }
 
@@ -425,8 +420,8 @@ get_addrinfo_from_etc_hosts(val_context_t * ctx,
                 (struct sockaddr_in *) malloc(sizeof(struct sockaddr_in));
             if (saddr4 == NULL) {
                 if (retval)
-                    freeaddrinfo(retval);
-                freeaddrinfo(ainfo);
+                    val_freeaddrinfo(retval);
+                val_freeaddrinfo(ainfo);
                 return EAI_MEMORY;
             }
             memset(saddr4, 0, sizeof(struct sockaddr_in));
@@ -446,8 +441,8 @@ get_addrinfo_from_etc_hosts(val_context_t * ctx,
                 malloc(sizeof(struct sockaddr_in6));
             if (saddr6 == NULL) {
                 if (retval)
-                    freeaddrinfo(retval);
-                freeaddrinfo(ainfo);
+                    val_freeaddrinfo(retval);
+                val_freeaddrinfo(ainfo);
                 return EAI_MEMORY;
             }
             memset(saddr6, 0, sizeof(struct sockaddr_in6));
@@ -463,7 +458,7 @@ get_addrinfo_from_etc_hosts(val_context_t * ctx,
         else {
             val_log(ctx, LOG_WARNING, 
                     "get_addrinfo_from_etc_hosts(): Unkown address type");
-            freeaddrinfo(ainfo);
+            val_freeaddrinfo(ainfo);
             continue;
         }
 
@@ -471,9 +466,9 @@ get_addrinfo_from_etc_hosts(val_context_t * ctx,
          * Expand the results based on servname and hints 
          */
         if ((ret = process_service_and_hints(servname, hints, &ainfo)) != 0) {
-            freeaddrinfo(ainfo);
+            val_freeaddrinfo(ainfo);
             if (retval)
-                freeaddrinfo(retval);
+                val_freeaddrinfo(retval);
             val_log(ctx, LOG_INFO, 
                     "get_addrinfo_from_etc_hosts(): Failed in process_service_and_hints()");
             return ret;
@@ -591,7 +586,7 @@ get_addrinfo_from_result(const val_context_t * ctx,
                     struct sockaddr_in *saddr4 = (struct sockaddr_in *)
                         malloc(sizeof(struct sockaddr_in));
                     if (saddr4 == NULL) {
-                        freeaddrinfo(ainfo);
+                        val_freeaddrinfo(ainfo);
                         return EAI_MEMORY;
                     }
                     val_log(ctx, LOG_DEBUG, "get_addrinfo_from_result(): rrset of type A found");
@@ -610,7 +605,7 @@ get_addrinfo_from_result(const val_context_t * ctx,
                     struct sockaddr_in6 *saddr6 = (struct sockaddr_in6 *)
                         malloc(sizeof(struct sockaddr_in6));
                     if (saddr6 == NULL) {
-                        freeaddrinfo(ainfo);
+                        val_freeaddrinfo(ainfo);
                         return EAI_MEMORY;
                     }
                     val_log(ctx, LOG_DEBUG, "get_addrinfo_from_result(): rrset of type AAAA found");
@@ -623,7 +618,7 @@ get_addrinfo_from_result(const val_context_t * ctx,
                 } 
 #endif
                 else {
-                    freeaddrinfo(ainfo);
+                    val_freeaddrinfo(ainfo);
                     rr = rr->rr_next;
                     continue;
                 }
@@ -640,7 +635,7 @@ get_addrinfo_from_result(const val_context_t * ctx,
                  */
                 if ((retval = process_service_and_hints(servname, hints, &ainfo))
                     != 0) {
-                    freeaddrinfo(ainfo);
+                    val_freeaddrinfo(ainfo);
                     val_log(ctx, LOG_INFO, 
                         "get_addrinfo_from_result(): Failed in process_service_and_hints()");
                     return retval;
@@ -701,7 +696,7 @@ get_addrinfo_from_result(const val_context_t * ctx,
  *            hints -- Hints to influence the return value.  Can be NULL.
  *              res -- A pointer to a variable of type (struct addrinfo *) to
  *                     hold the result.  The caller must free this return value
- *                     using freeaddrinfo().
+ *                     using val_freeaddrinfo().
  *
  * Returns: 0 on success and a non-zero value on error.
  *
@@ -817,13 +812,13 @@ _getaddrinfo_local(val_context_t * ctx, const char *nodename,
 {
     struct sockaddr_in  sa;
     struct addrinfo *ainfo4 = NULL;
-#if defined( WIN32 ) && !defined( LIBVAL_USE_WOCK )
+#if defined( WIN32 )
     size_t addrlen4 = sizeof(struct sockaddr_in);
 #endif
 #ifdef VAL_IPV6
     struct sockaddr_in6 sa6;
     struct addrinfo *ainfo6 = NULL;
-#if defined( WIN32 ) && !defined( LIBVAL_USE_WOCK )
+#if defined( WIN32 )
     size_t addrlen6 = sizeof(struct sockaddr_in6);
 #endif
 #endif
@@ -916,7 +911,7 @@ _getaddrinfo_local(val_context_t * ctx, const char *nodename,
 
         if ((retval = process_service_and_hints(servname, cur_hints, &ainfo4))
             != 0) {
-            freeaddrinfo(ainfo4);
+            val_freeaddrinfo(ainfo4);
             val_log(ctx, LOG_INFO, 
                     "val_getaddrinfo(): Failed in process_service_and_hints()");
             goto done;
@@ -970,7 +965,7 @@ _getaddrinfo_local(val_context_t * ctx, const char *nodename,
 
         if ((retval = process_service_and_hints(servname, cur_hints, &ainfo6))
             != 0) {
-            freeaddrinfo(ainfo6);
+            val_freeaddrinfo(ainfo6);
             val_log(ctx, LOG_INFO, 
                     "val_getaddrinfo(): Failed in process_service_and_hints()");
             goto done;
@@ -1050,7 +1045,7 @@ done:
  *                protocol is acceptable.
  *     [OUT] res: Points to a dynamically-allocated link list of addrinfo
  *                structures. The caller must free this return value
- *                using freeaddrinfo().
+ *                using val_freeaddrinfo().
  *
  *     Note that at least one of nodename or servname must be a non-NULL value.
  *
@@ -1235,11 +1230,7 @@ val_getnameinfo(val_context_t * context,
     if (ctx == NULL)
         return EAI_FAIL;;
 
-    val_log(ctx, LOG_DEBUG,
-            "val_getnameinfo called with host = %s, serv = %s",
-            host == NULL ? "(null)" : host,
-            serv == NULL ? "(null)" : serv);
-
+    val_log(ctx, LOG_DEBUG, "val_getnameinfo(): called");
 
     *val_status = VAL_UNTRUSTED_ANSWER;
 
@@ -1464,6 +1455,8 @@ int val_getaddrinfo_has_status(int rc) {
 
 #define VAL_GAI_DONE         0x0001
 
+#define VAL_AS_CANCEL_DONT_FREE 0x01000000
+
 struct val_gai_status_s {
     char             *nodename;
     char             *servname;
@@ -1481,23 +1474,31 @@ struct val_gai_status_s {
 };
 
 static void
-_free_vgai( val_gai_status *vgai )
+_cancel_vgai( val_gai_status *vgai, int flags )
 {
-    if (NULL == vgai)
-        return;
+    vgai->flags |= VAL_AS_CANCEL_DONT_FREE;
 
     if (NULL != vgai->inet_status) {
-        val_async_cancel(vgai->context, vgai->inet_status, 
-                         VAL_AS_CANCEL_NO_CALLBACKS);
+        val_async_cancel(vgai->context, vgai->inet_status, flags);
         vgai->inet_status = NULL;
     }
 
     if (NULL != vgai->inet6_status) {
-        val_async_cancel(vgai->context, vgai->inet6_status,
-                         VAL_AS_CANCEL_NO_CALLBACKS);
+        val_async_cancel(vgai->context, vgai->inet6_status, flags);
         vgai->inet6_status = NULL;
     }
 
+    vgai->flags &= ~VAL_AS_CANCEL_DONT_FREE;
+}
+
+static void
+_free_vgai( val_gai_status *vgai )
+{
+    if (NULL == vgai || vgai->flags & VAL_AS_CANCEL_DONT_FREE)
+        return;
+
+    _cancel_vgai( vgai, VAL_AS_CANCEL_NO_CALLBACKS);
+
     if (NULL != vgai->nodename) {
         free(vgai->nodename);
         vgai->nodename = NULL;
@@ -1509,13 +1510,22 @@ _free_vgai( val_gai_status *vgai )
     }
 
     if (NULL != vgai->res) {
-        freeaddrinfo(vgai->res);
+        val_freeaddrinfo(vgai->res);
         vgai->res = NULL;
     }
 
     free(vgai);
 }
 
+void
+val_getaddrinfo_cancel(val_gai_status *status, int flags)
+{
+    if (NULL == status)
+        return;
+    _cancel_vgai(status, flags);
+    _free_vgai(status);
+}
+
 /*
  * our internal callback for request completion, which in turns calls
  * original caller vgai_callback.
@@ -1597,7 +1607,10 @@ _vgai_async_callback(val_async_status *as, int event,
     if (NULL == vgai->callback) {
         val_log(ctx, LOG_DEBUG, "val_getaddrinfo async NULL callback!");
     } else {
-        (*vgai->callback)(vgai->callback_data, rc, vgai->res, vgai->val_status);
+        if (VAL_AS_EVENT_CANCELED == event)
+            gai_rc = VAL_AS_EVENT_CANCELED;
+        (*vgai->callback)(vgai->callback_data, gai_rc, vgai->res,
+                          vgai->val_status);
         /** let caller keep res structure */
         vgai->res = NULL;
     }
