@@ -6,7 +6,6 @@
  * DESCRIPTION
  * Parsing functions for some useful RR types
  */
-#include "validator/validator-config.h"
 #include "validator-internal.h"
 
 #include "val_parse.h"
@@ -172,6 +171,7 @@ val_parse_ds_string(char *dsstr, size_t dsstrlen,
     char           *dsptr = NULL;
     char           *cp;
     size_t          bufsize;
+    int             i;
 
     if (dsstr == NULL || ds_rdata == NULL)
         return VAL_BAD_ARGUMENT;
@@ -229,7 +229,7 @@ val_parse_ds_string(char *dsstr, size_t dsstrlen,
     }
 
     /* Convert the hex string to a byte string */
-    int  i = 0;
+    i = 0;
     while (dsptr < ep && i < bufsize) { 
         char hexdigit[3];
         memcpy(hexdigit, dsptr, 2);
