@@ -8,7 +8,6 @@
  * files and update the validator context.
  */
 
-#include "validator/validator-config.h"
 #include "validator-internal.h"
 
 #include "val_policy.h"
@@ -18,7 +17,14 @@
 #include "val_context.h"
 #include "val_assertion.h"
 #include "val_parse.h"
+
+#if !defined(WIN32) || defined(LIBVAL_CONFIGURED)
 #include "val_inline_conf.h"
+#else
+const char val_conf_inline_buf[] = "";
+const char root_hints_inline_buf[] = "";
+const char resolv_conf_inline_buf[] = "";
+#endif
 
 #ifdef ANDROID
 #include <sys/system_properties.h>
@@ -37,11 +43,11 @@
 #define STORE_POLICY_ENTRY_IN_LIST(pol, head) do {\
     policy_entry_t *prev, *cur, *next;\
     while (pol) {\
-        next = pol->next;\
         int name_len = wire_name_length(pol->zone_n);\
         /*\
          * Store according to decreasing zone name length \
          */\
+        next = pol->next;\
         prev = NULL;\
         for (cur = head; cur;\
             prev = cur, cur = cur->next)\
@@ -1812,6 +1818,7 @@ read_val_config_file(val_context_t * ctx, char *scope)
     char *label;
     char *newctxlab;
     struct val_query_chain *q;
+    char *logtarget = NULL;
    
     if (ctx == NULL)
         return VAL_BAD_ARGUMENT;
@@ -1919,7 +1926,6 @@ read_val_config_file(val_context_t * ctx, char *scope)
     }
 
     /* set the log target from environment */
-    char *logtarget = NULL;
     logtarget = getenv(VAL_LOG_TARGET);
     if (logtarget) {
         val_log_add_optarg_to_list(&ctx->val_log_targets, logtarget, 1);
