@@ -19,7 +19,6 @@
  * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
-#include "validator/validator-config.h"
 #include "validator-internal.h"
 
 #include "val_support.h"
@@ -64,7 +63,7 @@ void
 base32hex_encode(u_char * in, size_t inlen, u_char ** out,
                  size_t * outlen)
 {
-    u_char        base32hex[32] = "0123456789ABCDEFGHIJKLMNOPQRSTUV";
+    u_char        base32hex[] = "0123456789ABCDEFGHIJKLMNOPQRSTUV";
     u_char       *in_ch, *buf;
     u_char       *out_ch;
     u_char        padbuf[5];
@@ -239,7 +238,7 @@ add_to_qname_chain(struct qname_chain **qnames, const u_char * name_n)
 
 
 int
-qname_chain_first_name(struct qname_chain *qnames, const u_char * name_n)
+name_in_qname_chain(struct qname_chain *qnames, const u_char * name_n)
 {
     struct qname_chain *qc;
 
@@ -250,9 +249,10 @@ qname_chain_first_name(struct qname_chain *qnames, const u_char * name_n)
     while (qc != NULL && namecmp(qc->qnc_name_n, name_n) != 0)
         qc = qc->qnc_next;
 
-    return (qc != NULL && qc->qnc_next == NULL);
+    return (qc != NULL);
 }
 
+
 void
 free_qname_chain(struct qname_chain **qnames)
 {
@@ -1447,6 +1447,7 @@ copy_rrset_rec_list(struct rrset_rec *rr_set)
     return copy_set;
 }
 
+#if 0
 struct rrset_rec *
 copy_rrset_rec_list_in_zonecut(struct rrset_rec *rr_set, u_char *qname_n) 
 {
@@ -1481,6 +1482,7 @@ copy_rrset_rec_list_in_zonecut(struct rrset_rec *rr_set, u_char *qname_n)
     }
     return copy_set;
 }
+#endif
 
 /*
  *
